Package: datamedios
Type: Package
Title: Scraping Chilean Media
Version: 1.2.3
Authors@R: c(
  person(given = "Exequiel", family = "Trujillo", email = "exequiel.trujillo@ug.uchile.cl", role = c("aut", "cre", "cph", "fnd")),
  person(given = "Ismael", family = "Aguayo", email = "ismael.aguayo@ug.uchile.cl", role = c("aut", "fnd")),
  person(given = "Klaus", family = "Lehmann", email = "klehmann@fen.uchile.cl", role = c("aut", "fnd")),
  person(given = "Matias", family = "Gallardo", email = "matias.gallardo.v@ug.uchile.cl", role = c("ctb"))
  )
Maintainer: Exequiel Trujillo <exequiel.trujillo@ug.uchile.cl>
Description: A system for extracting news from Chilean media,
  specifically through Web Scapping from Chilean media. The package allows for news searches using search phrases and date filters,
  and returns the results in a structured format, ready for analysis.
  Additionally, it includes functions to clean the extracted data, visualize it, and
  store it in databases. All of this can be done automatically, facilitating the collection and analysis of relevant
  information from Chilean media.
License: MIT + file LICENSE
Encoding: UTF-8
Language: es-ES
Depends: R (>= 4.1)
Suggests: rcmdcheck
Imports: dplyr, httr, magrittr, jsonlite, utils, rlang, wordcloud2,
        tidytext, lubridate, rvest, stringr, xml2, purrr, DT, ggplot2,
        plotly, parallel, pbapply
URL: https://socialtec-cl.github.io/datamedios/,
        https://github.com/Socialtec-cl/datamedios
BugReports: https://github.com/Socialtec-cl/datamedios/issues
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-27 00:02:20 UTC; Compu
Author: Exequiel Trujillo [aut, cre, cph, fnd],
  Ismael Aguayo [aut, fnd],
  Klaus Lehmann [aut, fnd],
  Matias Gallardo [ctb]
Repository: CRAN
Date/Publication: 2026-04-27 02:10:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:15:51 UTC; unix

Package: datamedios
Type: Package
Title: Scraping Chilean Media
Version: 1.2.3
Authors@R: c(
  person(given = "Exequiel", family = "Trujillo", email = "exequiel.trujillo@ug.uchile.cl", role = c("aut", "cre", "cph", "fnd")),
  person(given = "Ismael", family = "Aguayo", email = "ismael.aguayo@ug.uchile.cl", role = c("aut", "fnd")),
  person(given = "Klaus", family = "Lehmann", email = "klehmann@fen.uchile.cl", role = c("aut", "fnd")),
  person(given = "Matias", family = "Gallardo", email = "matias.gallardo.v@ug.uchile.cl", role = c("ctb"))
  )
Maintainer: Exequiel Trujillo <exequiel.trujillo@ug.uchile.cl>
Description: A system for extracting news from Chilean media,
  specifically through Web Scapping from Chilean media. The package allows for news searches using search phrases and date filters,
  and returns the results in a structured format, ready for analysis.
  Additionally, it includes functions to clean the extracted data, visualize it, and
  store it in databases. All of this can be done automatically, facilitating the collection and analysis of relevant
  information from Chilean media.
License: MIT + file LICENSE
Encoding: UTF-8
Language: es-ES
Depends: R (>= 4.1)
Suggests: rcmdcheck
Imports: dplyr, httr, magrittr, jsonlite, utils, rlang, wordcloud2,
        tidytext, lubridate, rvest, stringr, xml2, purrr, DT, ggplot2,
        plotly, parallel, pbapply
URL: https://socialtec-cl.github.io/datamedios/,
        https://github.com/Socialtec-cl/datamedios
BugReports: https://github.com/Socialtec-cl/datamedios/issues
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-27 00:02:20 UTC; Compu
Author: Exequiel Trujillo [aut, cre, cph, fnd],
  Ismael Aguayo [aut, fnd],
  Klaus Lehmann [aut, fnd],
  Matias Gallardo [ctb]
Repository: CRAN
Date/Publication: 2026-04-27 02:10:02 UTC
Built: R 4.6.0; ; 2026-04-27 05:19:52 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(agregar_datos_unicos)
export(extraccion_parrafos)
export(extraer_noticias_fecha)
export(extraer_noticias_fecha_bbcl)
export(extraer_noticias_fecha_ciper)
export(extraer_noticias_fecha_emol)
export(extraer_noticias_max_res)
export(extraer_noticias_max_res_bbcl)
export(extraer_noticias_max_res_ciper)
export(extraer_noticias_max_res_emol)
export(grafico_comparacion_medios)
export(grafico_notas_fecha)
export(grafico_notas_por_mes)
export(init_req_bbcl)
export(init_req_emol)
export(iteracion_emol)
export(limpieza_notas)
export(parserFuentes)
export(tabla_frecuencia_palabras)
export(word_cloud)
import(dplyr)
import(pbapply)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(utils,URLencode)

# Paquete `datamedios` (Versión 1.2.3)

`datamedios` es un paquete de R diseñado para facilitar la extracción automatizada de noticias desde medios de comunicación chilenos, haciendo web scraping de medios chilenos. Este paquete permite realizar búsquedas de noticias y filtrarlas por rangos de fechas, entregando los resultados en un formato estructurado y listo para su análisis. Además, incluye funcionalidades para almacenar los datos extraídos en una base de datos de manera opcional ([documentación completa](https://socialtec-cl.github.io/datamedios/)).

[![DOI](https://zenodo.org/badge/901536889.svg)](https://doi.org/10.5281/zenodo.16990419)
------------------------------------------------------------------------

## 🔧 Instalación

Para instalarlo desde CRAN:

Instala `datamedios` desde CRAN:

``` r
install.packages("datamedios")
library(datamedios)
```

Alternativamente, para instalar el paquete desde GitHub, sigue los siguientes pasos:

1.  Asegúrate de tener instalado el paquete `devtools`. Si no lo tienes, instálalo ejecutando:

    ``` r
    install.packages("devtools")
    ```

2.  Instala `datamedios` desde el repositorio de GitHub:

    ``` r
    devtools::install_github("Socialtec-cl/datamedios")
    ```

-----------------------------------------------------------------------

## 🔄 Uso

### Función principal: `extraer_noticias_fecha`

Esta función permite filtrar noticias por un rango de fechas específico, además de aplicar una frase de búsqueda.

#### **Parámetros:**

-   **`search_query`**: Frase de búsqueda (obligatorio excepto para Ciper).
-   **`fecha_inicio`**: Fecha de inicio en formato `"YYYY-MM-DD"` (obligatorio).
-   **`fecha_fin`**: Fecha de fin en formato `"YYYY-MM-DD"` (obligatorio).
-   **`subir_a_bd`**: Por defecto está seteado en TRUE, para los tests lo dejamos en FALSE (opcional).
-   **`fuentes`**:Es un string con las fuentes a extraer. Puede ser:
    -   `"todas"`: Todas las fuentes disponibles.
    -   `"bbcl"`: BíoBío como única fuente
    -   `"emol-todas"`: Búsqueda simultánea en las fuentes "emol", "guioteca" y "mediosregionales".
    -   `"emol"`: Noticias no pagas de El Mercurio.
    -   `"mediosregionales"`: Noticias de los diarios regionales propiedad de El Mercurio.
    -   `"guioteca"`: Guías especializadas de Emol.
    -   `"ciper"`: Publicaciones de Ciper Chile.
    -   **O una combinación,** solo separando medios por comas, por ejemplo: `fuentes = "mediosregionales, bbcl"`.

#### **Valor devuelto:**

Un `data.frame` con las siguientes columnas:

-   **`ID`**: Identificador de la noticia.
-   **`titulo`**: Título de la noticia.
-   **`contenido`**: Contenido completo.
-   **`contenido limpio`**: Resumen o extracto.
-   **`url`**: Enlace a la noticia.
-   **`url_imagen`**: Url con la imagen principal de la noticia.
-   **`autor`**: Autor de la noticia.
-   **`fecha`**: Fecha de publicación (formato 'YYYY-MM-DD').
-   **`resumen`**: Resumen de la IA o bajada de la nota, según disponibilidad.
-   **`search_query`**: Palabra o frase de búsqueda por la que se obtuvo los datos.
-   **`medio`**: Medio al que corresponde la noticia.
-   **`temas`**: Categorías de la noticia.

#### **Ejemplo de uso:**

``` r
# Buscar noticias entre el 18 de octubre de 2019 al 18 de octubre de 2020
noticias <- extraer_noticias_fecha("estallido social", "2019-10-18", "2020-10-18", fuentes = "bbcl, emol, mediosregionales")
```

------------------------------------------------------------------------

### Función secundaria: `extraer_noticias_max_res`

Esta función permite obtener noticias desde medios chilenos utilizando una frase de búsqueda. Además, puedes limitar el número de resultados a extraer.

#### **Parámetros:**

-   **`search_query`**: Frase de búsqueda (obligatorio excepto para Ciper).
-   **`max_results`**: Máximo número de resultados a extraer (opcional).
-   **`subir_a_bd`**: Por defecto está seteado en TRUE, pero para los tests lo dejamos en FALSE (opcional).
-   **`fuentes`**:Es un string con las fuentes a extraer. Funciona de la misma forma que en extraer_noticias_fecha. Por defecto extrae de todas las fuentes (opcional).

#### **Valor devuelto:**

Un `data.frame` similar al de `extraer_noticias_fecha`, pero filtrado por `max_results`.

#### **Ejemplo de uso:**

``` r
# Buscar noticias relacionadas con "inteligencia artificial"
noticias <- extraer_noticias_max_res("inteligencia artificial", max_results = 100, fuentes= "bbcl")
```

## 🔖 Documentación

**Adicionalmente, el paquete incluye funciones de limpieza, manejo y visualización de las noticias extraídas. Para visitar la documentación de todos los códigos haga click [aquí](https://socialtec-cl.github.io/datamedios/reference/index.html).**

## 📊 Dependencias

Este paquete utiliza las siguientes dependencias de R para su correcto funcionamiento:

-   **`dplyr`**: Manipulación y análisis de datos.

-   **`httr`**: Realización de solicitudes HTTP para interactuar con APIs.

-   **`magrittr`**: Uso de operadores como `%>%` para flujos de trabajo más legibles.

-   **`jsonlite`**: Procesamiento y conversión de datos en formato JSON.

-   **`utils`**: Funciones utilitarias básicas incluidas en R.

-   **`rlang`**: Herramientas para manipulación de expresiones.

-   **`wordcloud2`**: Generación de nubes de palabras interactivas.

-   **`tidytext`**: Análisis de texto basado en datos ordenados.

-   **`lubridate`**: Manejo y análisis de datos temporales.

-   **`rvest`**: Web scraping de páginas HTML.

-   **`stringr`**: Manejo y manipulación de cadenas de texto.

-   **`xml2`**: Lectura y manejo de datos en formato XML.

-   **`purrr`**: Programación funcional con listas y vectores.

-   **`DT`**: Generación de tablas interactivas en HTML desde R.

-   **`ggplot2`**: Creación de gráficos sofisticados y personalizados basados en la gramática de los gráficos.

-   **`plotly`**: Creación de gráficos interactivos y visualizaciones dinámicas.

-   **`parallel`**: Ejecución paralela multinucleo.

-   **`pbapply`**: Barra de progreso.

------------------------------------------------------------------------

## 🗂️ Licencia

`datamedios` está bajo la Licencia MIT. Consulta el archivo `LICENSE` para más detalles.

------------------------------------------------------------------------

## 👤 Autoría

Este paquete fue desarrollado por:

-   **Exequiel Trujillo** (contacto: [exequiel.trujillo\@ug.uchile.cl](mailto:exequiel.trujillo@ug.uchile.cl))
-   **Ismael Aguayo** (contacto: [ismael.aguayo\@ug.uchile.cl](mailto:ismael.aguayo@ug.uchile.cl))
-   **Klaus Lehmann** (contacto: [klehmann\@fen.uchile.cl](mailto:klehmann@fen.uchile.cl))

Contribuciones adicionales de:

-   **Matias Gallardo** (contacto: [matias.gallardo.v\@ug.uchile.cl](mailto:matias.gallardo.v@ug.uchile.cl))

------------------------------------------------------------------------

#' Inicializa una solicitud a emol.com y retorna el primer caso de busqueda
#'
#' Esta funcion permite realizar una consulta inicial a emol.com utilizando una frase de busqueda.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param fuentes Un string donde se ponen las fuentes de emol a consultar
#' @return Un dataframe con el primer caso de la busqueda.
#' @export
#' @examples
#' \dontrun{
#' primer_caso <- init_req_emol("Boric", fuentes="emol")
#' }

init_req_emol <- function(search_query, fuentes="emol-todas") {
  # Validamos el parametro
  if (missing(search_query) || !is.character(search_query)) {
    stop("Debe proporcionar una frase de busqueda como texto.")
  }
  # Hay que validar el formato del string fuente

  # Parametros de configuracion y adicionales
  total_articles <- 0  # Basado en `data_initial$hits$total`
  batch_size <- 10
  data_initial <- list()
  patronBusqueda <- ""

  if(fuentes=="emol-todas"){
    patronBusqueda <- "emol,mediosregionales,guioteca"
  } else {
    fuentesParsed <- parserFuentes(fuentes)
    if(length(fuentesParsed) > 0){
      patronBusqueda <- paste0(fuentesParsed, collapse = ",")
    } else {
      message("Error al parsear")
    }
  }


  # URL inicial
  url_initial <- paste0(
    "https://newsapi.ecn.cl/NewsApi/emol/buscador/",
    patronBusqueda,
    "?q=",URLencode(search_query),
    "&size=10&from=0"
  )

  # Solicitud inicial
  response_initial <- httr::GET(url_initial)
  if (response_initial$status_code == 200) {
    data_initial <- httr::content(response_initial, "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON(flatten = TRUE)

    total_articles = data_initial$hits$total
    message(paste("Total de resultados disponibles en", patronBusqueda, "para la busqueda:", total_articles))

    if (!is.null(data_initial$hits$hits) && length(data_initial$hits$hits) > 0) {
      # Extraemos el primer caso
      # first_case <- as.data.frame(data_initial$hits$hits[1, ])
      # Extraemos máximo los 10 primeros casos
      first_case <- as.data.frame(data_initial$hits$hits)

      # Agregamos la columna "total"
      first_case$total <- total_articles

      # Retornamos el primer caso
      return(first_case)
    } else {
      stop("No se encontraron notas en la respuesta inicial.")
    }
  } else {
    stop("Error al realizar la solicitud inicial. Codigo de estado: ", response_initial$status_code)
  }
}

#' Extraccion de noticias de Emol.com
#'
#' Esta funcion permite extraer noticias de las fuentes de Emol, tanto de las
#' noticias no pagas de emol, como de guioteca y los medios regionales de El Mercurio
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param max_results Numero maximo de resultados a extraer (opcional, por defecto todos).
#' @param fuente Fuente de emol para iterar (obligatoria).
#' @return Un dataframe con las noticias extraidas.
#' @examples
#' \dontrun{
#' noticias <- extraer_noticias_max_res_emol("inteligencia artificial", "2025-01-01",
#' "2025-02-24", fuente="mediosregionales")
#' }
#' @export
extraer_noticias_max_res_emol <- function(search_query, max_results=NULL, fuente){
  # Validamos los parametros
  if (missing(search_query) || !is.character(search_query)) {
    stop("Debe proporcionar una frase de busqueda valida como texto.")
  }
  if (!is.null(max_results) && (!is.numeric(max_results) || max_results <= 0)) {
    stop("max_results debe ser un numero entero positivo o NULL.")
  }

  # Funcion auxiliar para crear dataframe vacio
  create_empty_df <- function() {
    data.frame(
      ID = character(),
      titulo = character(),
      contenido = character(),
      contenido_limpio = character(),
      url = character(),
      url_imagen = character(),
      autor = character(),
      fecha = character(),
      temas = I(list()),
      resumen = character(),
      search_query = character(),
      medio = character(),
      stringsAsFactors = FALSE
    )
  }

  # Obtener la respuesta inicial
  respuesta_inicial <- tryCatch({
    init_req_emol(search_query, fuentes = fuente)
  }, error = function(e) {
    message("Error al obtener la respuesta inicial: ", e$message)
    return(NULL)
  })

  # Verificar si hay resultados en la respuesta inicial
  if (is.null(respuesta_inicial) || nrow(respuesta_inicial) == 0) {
    message(paste("No se encontraron resultados iniciales para la fuente:", fuente))
    return(create_empty_df())
  }

  # Calcular total de resultados
  total_results <- tryCatch({
    as.numeric(respuesta_inicial$total[1])
  }, error = function(e) {
    message("No se pudo convertir total de resultados: ", e$message)
    NA_real_
  })

  # Verificar total de resultados
  if (is.na(total_results) || total_results <= 0) {
    message("No se pudo determinar el numero total de resultados.")
    return(create_empty_df())
  }

  # Determinar numero maximo de resultados a extraer
  if (is.null(max_results) || max_results > total_results) {
    max_results <- total_results
  }

  # Inicializar variables para el bucle
  all_data <- create_empty_df()
  offset <- 0

  repeat {
    # Obtener los datos de la pagina actual
    datos_pagina <- tryCatch({
      iteracion_emol(search_query, page = offset, fuentes = fuente)
    }, error = function(e) {
      message("Error al obtener la pagina ", offset, " de la fuente ", fuente,": ", e$message)
      return(NULL)
    })

    # Verificar si hay datos
    if (is.null(datos_pagina) || nrow(datos_pagina) == 0) {
      message("No hay mas resultados o se produjo un error. Finalizando extraccion.")
      break
    }

    # Agregar los datos al dataframe final
    all_data <- tryCatch({
      rbind(all_data, datos_pagina)
    }, error = function(e) {
      message("Error al combinar datos: ", e$message)
      all_data
    })

    # Verificar si alcanzamos el numero maximo de resultados
    if (nrow(all_data) >= max_results) {
      all_data <- all_data[1:max_results, ]
      break
    }

    # Incrementar el offset para la siguiente iteracion
    offset <- offset + 1

    # Opcional: agregar un retraso para no sobrecargar la API
    # Sys.sleep(0.5)
  }

  ##################################################################################

  #### PROCESAMIENTO DF FINAL ####

  # Verificamos si hay datos para procesar
  if (is.null(all_data) || nrow(all_data) == 0) {
    message(paste("No se encontraron datos para la fuente:", fuente, "en el rango de fechas especificado."))
    return(create_empty_df())
  }

  # Verificamos que el dataframe tenga todas las columnas necesarias para el nucleo
  # Las columnas para helpers seran verificadas implicitamente por los helpers
  required_columns <- c("_id", "_source.titulo", "_source.texto", "_source.permalink")
  missing_columns <- required_columns[!required_columns %in% names(all_data)]

  if (length(missing_columns) > 0) {
    message(paste("Faltan columnas base necesarias en los datos:", paste(missing_columns, collapse=", ")))
    return(create_empty_df())
  }

  all_data$search_query <- tolower(search_query)
  all_data$medio <- fuente

  fecha_source_col_name <- if (fuente == "guioteca") {
    "_source.fechaModificacion"
  } else if ("_source.fechaPublicacion" %in% names(all_data)) {
    "_source.fechaPublicacion"
  } else if ("_source.fechaModificacion" %in% names(all_data)) { # Anadido chequeo de existencia
    "_source.fechaModificacion"
  } else {
    message(paste("Advertencia: No se encontro columna de fecha '_source.fechaPublicacion' ni '_source.fechaModificacion' para fuente:", fuente, ". Las fechas seran NA."))
    NA_character_ # Para que rlang::sym no falle si no existe
  }

  processed_data <- tryCatch({
    # Antes de procesar, anadimos columnas que podrian faltar para los helpers, con NAs
    # Esto evita errores si una columna esperada por un helper no existe en all_data para alguna fuente
    cols_for_helpers <- c(
      "_source.tablas.tablaMedios" = NA, # Tipo de NA dependera, pero dplyr lo maneja
      "_source.imagen" = NA_character_,
      "_source.autor" = NA_character_,
      "_source.bajada" = NA, # Puede ser lista o df, NA es generico
      "_source.seccion" = NA_character_,
      "_source.subSeccion" = NA_character_,
      "_source.temas" = NA # Puede ser lista o df
    )

    for(col_name in names(cols_for_helpers)){
      if(!col_name %in% names(all_data)){
        all_data[[col_name]] <- cols_for_helpers[[col_name]]
      }
    }

    # Asegurar que la columna de fecha exista, aunque sea con NAs
    if (!is.na(fecha_source_col_name) && !fecha_source_col_name %in% names(all_data)) {
      all_data[[fecha_source_col_name]] <- NA_character_
    }


    all_data %>%
      dplyr::rowwise() %>%
      dplyr::mutate(
        ID = paste0(`_id`, "-e"),
        titulo = dplyr::coalesce(as.character(`_source.titulo`), NA_character_), # Asegurar character y manejar NULL/NA
        contenido = dplyr::coalesce(as.character(`_source.texto`), NA_character_),
        contenido_limpio = NA_character_, # Se mantiene como NA, se llenara despues
        url = dplyr::coalesce(as.character(`_source.permalink`), NA_character_),

        url_imagen = helper_extraer_url_imagen(
          `_source.tablas.tablaMedios`,
          `_source.imagen`,
          fuente # Pasamos la variable 'fuente' del entorno de la funcion principal
        ),

        autor = dplyr::coalesce(
          if (fuente == "guioteca") "guioteca" else as.character(`_source.autor`),
          NA_character_
        ),

        fecha = if (!is.na(fecha_source_col_name)) {
          as.character(as.Date(dplyr::coalesce(!!rlang::sym(fecha_source_col_name), NA_character_)))
        } else {
          NA_character_
        },

        resumen = helper_extraer_resumen(`_source.bajada`),

        # Para temas, pasamos las columnas relevantes al helper
        # El helper devolvera una lista, que es lo que se espera para una list-column
        temas = helper_extraer_temas(
          `_source.seccion`,
          `_source.subSeccion`,
          `_source.temas`,
          fuente # Pasamos la variable 'fuente'
        )
      ) %>%
      dplyr::ungroup() %>%
      dplyr::select(
        ID, titulo, contenido, contenido_limpio, url,
        url_imagen, autor, fecha, temas, resumen,
        search_query, medio
      )

  }, error = function(e) {
    message(paste("--------------------------------------------------------------------"))
    message(paste("ERROR CRITICO durante el procesamiento de datos para la fuente:", fuente))
    message(paste("Mensaje de error:", e$message))

    # Intentar obtener el indice de la fila que pudo causar el error si es un error de dplyr
    # Esto es una heuristica y puede no funcionar siempre
    error_msg_str <- as.character(e)
    if (grepl("problem with `mutate()` input", error_msg_str, ignore.case = TRUE) || grepl("evaluation error", error_msg_str, ignore.case = TRUE)) {
      message("El error parece haber ocurrido durante una operacion de mutate por fila.")
      message("Es dificil identificar la fila exacta sin mas informacion del error de dplyr.")
      message("Se recomienda inspeccionar 'all_data_problematic_sample.rds' (si se crea).")
    }

    message("Estructura de 'all_data' (primeras 5 filas y columnas relevantes):")
    if (nrow(all_data) > 0) {
      cols_to_print <- intersect(c("_id", "_source.titulo", "_source.permalink", "_source.bajada", "_source.tablas.tablaMedios", "_source.imagen", "_source.seccion", "_source.subSeccion", "_source.temas", fecha_source_col_name), names(all_data))
      if (length(cols_to_print) > 0) {
        try(print(utils::head(all_data[, cols_to_print, drop = FALSE], 5)), silent = TRUE)
        message("Estructura detallada (str) de columnas potencialmente problematicas de la primera fila:")
        if ("_source.bajada" %in% names(all_data)) try(print(utils::str(all_data$`_source.bajada`[[1]])), silent = TRUE)
        if ("_source.tablas.tablaMedios" %in% names(all_data)) try(print(utils::str(all_data$`_source.tablas.tablaMedios`[[1]])), silent = TRUE)
        if ("_source.temas" %in% names(all_data)) try(print(utils::str(all_data$`_source.temas`[[1]])), silent = TRUE)
      } else {
        message("No se pudieron encontrar columnas relevantes para imprimir de 'all_data'.")
      }

      # Guardar una muestra de all_data para depuracion externa
      # Guardamos solo las primeras 100 filas o todas si son menos
      sample_size <- min(nrow(all_data), 100)
      all_data_sample <- all_data[1:sample_size, ]
      # OJO: esto guarda en el directorio de trabajo actual. Considerar una ruta mas especifica si es necesario.
      problem_file_path <- paste0("all_data_problematic_sample_FUENTE_",gsub("[^A-Za-z0-9]", "_", fuente),".rds")
      try(saveRDS(all_data_sample, file = problem_file_path), silent = TRUE)
      message(paste("Se ha guardado una muestra de datos problematicos en:", problem_file_path))
      message(paste("Puedes cargarla con: readRDS('",problem_file_path,"')",sep=""))

    } else {
      message("'all_data' esta vacio en el momento del error.")
    }
    message(paste("--------------------------------------------------------------------"))

    return(create_empty_df()) # Devuelve dataframe vacio como antes
  })

  return(processed_data)
}

#' Grafico de comparacion de medios por periodo (Interactivo)
#'
#' Esta funcion genera un grafico interactivo que compara la cantidad de publicaciones
#' entre diferentes medios de medios, agrupadas por dia o por mes, con opcion de tema dark o light.
#'
#' @param datos Data frame con los datos procesados, que debe incluir las columnas `fecha` y `medio`.
#' @param titulo Texto que aparecera en el titulo del grafico.
#' @param fecha_inicio Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @param fecha_fin Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @param medios Vector de strings con las medios a comparar. Si es NULL, usa todas las medios disponibles.
#' @param agrupar_por Cadena de texto que especifica el periodo de agrupacion.
#'   Valores validos son `"day"` (por defecto) o `"month"`.
#' @param tema Tema del grafico. Valores validos son `"light"` (por defecto) o `"dark"`.
#' @param tipo_grafico Tipo de visualizacion. Valores validos son `"lineas"` (por defecto) o `"barras"`.
#' @return Un grafico plotly interactivo que muestra la comparacion de publicaciones por medio y periodo.
#' @examples
#' \dontrun{
#' # Comparar todas las medios por mes
#' datos <- extraer_noticias_fecha("delincuencia", "2024-01-01", "2025-01-01", subir_a_bd = FALSE)
#' grafico_comparacion_medios(datos, titulo = "Cobertura de Delincuencia por Medio",
#'                             agrupar_por = "month", tema = "dark")
#'
#' # Comparar medios especificas por dia
#' grafico_comparacion_medios(datos, titulo = "Comparacion BBCl vs emol",
#'                             medios = c("bbcl", "emol"),
#'                             fecha_inicio = "2024-06-01", fecha_fin = "2024-06-30",
#'                             agrupar_por = "day", tipo_grafico = "barras")
#' }
#'
#'
#' @importFrom stats setNames
#' @export
grafico_comparacion_medios <- function(datos, titulo, fecha_inicio = NULL, fecha_fin = NULL,
                                        medios = NULL, agrupar_por = "day", tema = "light",
                                        tipo_grafico = "lineas") {

  # Cargar librerias necesarias
  if (!requireNamespace("plotly", quietly = TRUE)) {
    stop("el paquete 'plotly' es necesario para esta funcion. Instalalo con: install.packages('plotly')")
  }
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("el paquete 'dplyr' es necesario para esta funcion. Instalalo con: install.packages('dplyr')")
  }
  if (!requireNamespace("lubridate", quietly = TRUE)) {
    stop("el paquete 'lubridate' es necesario para esta funcion. Instalalo con: install.packages('lubridate')")
  }

  # --- 1. Validacion de entradas ---

  # Validar que 'datos' sea un data frame
  if (!is.data.frame(datos)) {
    stop("'datos' debe ser un data frame.")
  }

  # Validar que 'fecha' y 'medio' existan en los datos
  if (!"fecha" %in% colnames(datos)) {
    stop("'datos' debe contener la columna 'fecha'.")
  }
  if (!"medio" %in% colnames(datos)) {
    stop("'datos' debe contener la columna 'medio'.")
  }

  # Validar que 'titulo' sea texto
  if (!is.character(titulo) || length(titulo) != 1) {
    stop("'titulo' debe ser una unica cadena de texto (string).")
  }

  # Validar el parametro 'agrupar_por'
  if (!agrupar_por %in% c("month", "day")) {
    stop("'agrupar_por' debe ser 'month' o 'day'.")
  }

  # Validar el parametro 'tema'
  if (!tema %in% c("light", "dark")) {
    stop("'tema' debe ser 'light' o 'dark'.")
  }

  # Validar el parametro 'tipo_grafico'
  if (!tipo_grafico %in% c("lineas", "barras")) {
    stop("'tipo_grafico' debe ser 'lineas' o 'barras'.")
  }

  # --- 2. Preparacion y Filtrado de Datos ---

  # Convertir 'fecha' a formato fecha
  datos_proc <- datos %>%
    dplyr::mutate(fecha = as.Date(fecha, format = "%Y-%m-%d"))

  # Filtrar por medios si se especifican
  if (!is.null(medios)) {
    medios_disponibles <- unique(datos_proc$medio)
    medios_no_encontradas <- setdiff(medios, medios_disponibles)

    if (length(medios_no_encontradas) > 0) {
      warning(paste("Las siguientes medios no se encontraron en los datos:",
                    paste(medios_no_encontradas, collapse = ", ")))
    }

    medios_validas <- intersect(medios, medios_disponibles)
    if (length(medios_validas) == 0) {
      stop("Ninguna de las medios especificadas se encontro en los datos.")
    }

    datos_proc <- datos_proc %>%
      dplyr::filter(medio %in% medios_validas)
  }

  # Filtrar por rango de fechas si se especifican
  if (!is.null(fecha_inicio) || !is.null(fecha_fin)) {
    if (!is.null(fecha_inicio)) {
      fecha_inicio <- as.Date(fecha_inicio, format = "%Y-%m-%d")
    } else {
      fecha_inicio <- min(datos_proc$fecha, na.rm = TRUE)
    }
    if (!is.null(fecha_fin)) {
      fecha_fin <- as.Date(fecha_fin, format = "%Y-%m-%d")
    } else {
      fecha_fin <- max(datos_proc$fecha, na.rm = TRUE)
    }

    # Validar que las fechas sean validas
    if (is.na(fecha_inicio) || is.na(fecha_fin)) {
      stop("Las fechas de inicio/fin no son validas o no hay fechas en los datos.")
    }
    if (fecha_inicio > fecha_fin) {
      stop("'fecha_inicio' debe ser anterior o igual a 'fecha_fin'.")
    }

    # Filtrar datos por rango
    datos_filtrados <- datos_proc %>%
      dplyr::filter(fecha >= fecha_inicio & fecha <= fecha_fin)
  } else {
    datos_filtrados <- datos_proc
  }

  # Verificar si hay datos despues del filtrado
  if (nrow(datos_filtrados) == 0) {
    stop("No hay datos en el rango de fechas y medios seleccionadas.")
  }

  # --- 3. Configuracion de Colores y Paletas ---

  medios_unicas <- sort(unique(datos_filtrados$medio))
  n_medios <- length(medios_unicas)

  if (tema == "dark") {
    colores_tema <- list(
      fondo = "#1a1a1a",
      texto = "#ffffff",
      grilla = "#404040"
    )
    # Paleta de colores vibrantes para tema dark
    paleta_medios <- c("#00d4ff", "#ff6b35", "#39ff14", "#ff1493", "#ffd700",
                        "#9370db", "#00fa9a", "#ff4500", "#1e90ff", "#ff69b4")
  } else {
    colores_tema <- list(
      fondo = "#ffffff",
      texto = "#333333",
      grilla = "#e0e0e0"
    )
    # Paleta de colores contrastantes para tema light
    paleta_medios <- c("#1f77b4", "#d62728", "#2ca02c", "#ff7f0e", "#9467bd",
                        "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf")
  }

  # Asignar colores a cada medio
  colores_medios <- setNames(paleta_medios[1:n_medios], medios_unicas)

  # --- 4. Agregacion de Datos por medio ---

  if (agrupar_por == "month") {
    # Agrupar datos por mes y medio
    publicaciones_agrupadas <- datos_filtrados %>%
      dplyr::mutate(fecha_grupo = lubridate::floor_date(fecha, "month")) %>%
      dplyr::group_by(fecha_grupo, medio) %>%
      dplyr::summarise(cantidad = dplyr::n(), .groups = "drop") %>%
      dplyr::arrange(fecha_grupo, medio)
  } else {
    # Agrupar datos por dia y medio
    publicaciones_agrupadas <- datos_filtrados %>%
      dplyr::group_by(fecha, medio) %>%
      dplyr::rename(fecha_grupo = fecha) %>%
      dplyr::summarise(cantidad = dplyr::n(), .groups = "drop") %>%
      dplyr::arrange(fecha_grupo, medio)
  }

  # Completar fechas faltantes con 0 para cada medio
  fechas_completas <- seq(min(publicaciones_agrupadas$fecha_grupo),
                          max(publicaciones_agrupadas$fecha_grupo),
                          by = if(agrupar_por == "month") "month" else "day")

  grid_completo <- expand.grid(fecha_grupo = fechas_completas,
                               medio = medios_unicas,
                               stringsAsFactors = FALSE)

  publicaciones_agrupadas <- grid_completo %>%
    dplyr::left_join(publicaciones_agrupadas, by = c("fecha_grupo", "medio")) %>%
    dplyr::mutate(cantidad = ifelse(is.na(cantidad), 0, cantidad)) %>%
    dplyr::arrange(fecha_grupo, medio)

  # --- 5. Creacion de Tooltips Personalizados ---

  if (agrupar_por == "month") {
    publicaciones_agrupadas <- publicaciones_agrupadas %>%
      dplyr::mutate(
        fecha_texto = format(fecha_grupo, "%Y-%m"),
        hover_text = paste0("medio: ", medio, "<br>",
                            "Fecha: ", fecha_texto, "<br>",
                            "Cantidad: ", cantidad)
      )
  } else {
    publicaciones_agrupadas <- publicaciones_agrupadas %>%
      dplyr::mutate(
        fecha_texto = format(fecha_grupo, "%Y-%m-%d"),
        hover_text = paste0("medio: ", medio, "<br>",
                            "Fecha: ", fecha_texto, "<br>",
                            "Cantidad: ", cantidad)
      )
  }

  # --- 6. Creacion del Grafico con Plotly Nativo ---

  # Crear grafico base
  grafico_plotly <- plotly::plot_ly()

  if (tipo_grafico == "lineas") {
    # Agregar lineas para cada medio
    for (medio_actual in medios_unicas) {
      datos_medio <- publicaciones_agrupadas %>%
        dplyr::filter(medio == medio_actual)

      grafico_plotly <- grafico_plotly %>%
        plotly::add_trace(
          data = datos_medio,
          x = ~fecha_grupo,
          y = ~cantidad,
          type = "scatter",
          mode = "lines+markers",
          name = medio_actual,
          line = list(color = colores_medios[medio_actual], width = 2.5),
          marker = list(color = colores_medios[medio_actual], size = 6),
          text = ~hover_text,
          hovertemplate = "%{text}<extra></extra>"
        )
    }
  } else { # tipo_grafico == "barras"
    # Crear grafico de barras agrupadas
    for (medio_actual in medios_unicas) {
      datos_medio <- publicaciones_agrupadas %>%
        dplyr::filter(medio == medio_actual)

      grafico_plotly <- grafico_plotly %>%
        plotly::add_trace(
          data = datos_medio,
          x = ~fecha_grupo,
          y = ~cantidad,
          type = "bar",
          name = medio_actual,
          marker = list(color = colores_medios[medio_actual], opacity = 0.8),
          text = ~hover_text,
          hovertemplate = "%{text}<extra></extra>"
        )
    }
  }

  # --- 7. Configuracion del Layout ---

  grafico_plotly <- grafico_plotly %>%
    plotly::layout(
      title = list(
        text = titulo,
        font = list(color = colores_tema$texto, size = 16, family = "Arial"),
        x = 0.5
      ),
      xaxis = list(
        title = list(text = "Fecha", font = list(color = colores_tema$texto)),
        tickfont = list(color = colores_tema$texto),
        gridcolor = colores_tema$grilla,
        zerolinecolor = colores_tema$grilla
      ),
      yaxis = list(
        title = list(text = "Cantidad de Noticias", font = list(color = colores_tema$texto)),
        tickfont = list(color = colores_tema$texto),
        gridcolor = colores_tema$grilla,
        zerolinecolor = colores_tema$grilla
      ),
      plot_bgcolor = colores_tema$fondo,
      paper_bgcolor = colores_tema$fondo,
      font = list(color = colores_tema$texto, family = "Arial", size = 12),
      hovermode = "x unified",
      legend = list(
        font = list(color = colores_tema$texto),
        bgcolor = "rgba(0,0,0,0)",
        bordercolor = colores_tema$texto,
        borderwidth = 1
      ),
      barmode = if(tipo_grafico == "barras") "group" else NULL
    ) %>%
    plotly::config(
      displayModeBar = TRUE,
      modeBarButtonsToRemove = c("pan2d", "select2d", "lasso2d", "autoScale2d"),
      displaylogo = FALSE,
      toImageButtonOptions = list(
        format = "png",
        filename = paste0("comparacion_medios_", gsub(" ", "_", tolower(titulo))),
        height = 500,
        width = 900,
        scale = 2
      )
    )

  # --- 8. Devolver el Grafico Interactivo ---
  return(grafico_plotly)
}

#' Scrapea el cuerpo de las noticias desde una lista de URLs de Ciper
#' 
#' Esta funcion itera sobre un dataframe de noticias, visita la URL de cada una,
#' y extrae el texto principal usando un selector CSS especifico para Ciper.
#' Admite ejecucion en paralelo para acelerar el proceso.
#'
#' @param news_df Un dataframe que debe contener una columna ''url''.
#' @param parallel Booleano que indica si se debe usar procesamiento en paralelo. Por defecto es TRUE.
#' @return El dataframe de entrada con la columna ''contenido_limpio'' rellenada con el texto scrapeado.
#' @keywords internal
#' @import pbapply
scrape_body_ciper <- function(news_df, parallel = TRUE) {
  # --- Validacion de entradas ---
  if (!is.data.frame(news_df) || !"url" %in% names(news_df)) {
    stop("El \'news_df\' debe ser un dataframe con una columna \'url\".")
  }
  if (nrow(news_df) == 0) {
    message("No hay articulos para scrapear.")
    return(news_df)
  }

  # --- Inicializar columna de contenido ---
  if (!"contenido_limpio" %in% names(news_df)) {
    news_df$contenido_limpio <- NA_character_
  }

  # --- Logica de Scrapeo ---
  if (parallel && requireNamespace("parallel", quietly = TRUE) && requireNamespace("pbapply", quietly = TRUE)) {
    # --- EJECUCION EN PARALELO ---
    num_cores <- parallel::detectCores() - 1
    if (num_cores < 1) num_cores <- 1

    num_cores <- min(num_cores, 2)# CRAN exige 2 cores maximo

    cl <- parallel::makeCluster(num_cores)

    # Exportar paquetes y variables necesarios a los workers
    parallel::clusterEvalQ(cl, {
      library(httr)
      library(rvest)
      library(stringr)
    })

    urls_to_process <- news_df$url
    message(paste("Iniciando scrapeo para", length(urls_to_process), "articulos..."))

    # Funcion que se ejecutara en cada worker
    scrape_single_url <- function(url) {
      content_selector <- "div.col-lg-9 p.texto-nota"
      user_agent <- "datamedios R package scraper (https://github.com/exetrujillo/datamedios)"
      max_retries <- 3
      retry_delay <- 5

      for (attempt in 1:max_retries) {
        response <- tryCatch(
          httr::GET(url, httr::user_agent(user_agent), httr::timeout(30)),
          error = function(e) e
        )

        if (!inherits(response, "error") && httr::status_code(response) < 500) {
          break
        }
        Sys.sleep(retry_delay)
      }

      if (inherits(response, "error")) {
        return(paste("REQUEST_ERROR:", response$message))
      }

      if (httr::status_code(response) == 200) {
        page_html <- rvest::read_html(response)
        content_nodes <- rvest::html_nodes(page_html, content_selector)
        if (length(content_nodes) > 0) {
          text_pieces <- rvest::html_text(content_nodes)
          return(stringr::str_squish(paste(text_pieces, collapse = "\n\n")))
        } else {
          return("")
        }
      } else {
        return(paste("HTTP_ERROR:", httr::status_code(response)))
      }
    }

    # Ejecutar en paralelo con barra de progreso
    results_list <- pbapply::pblapply(cl = cl, X = urls_to_process, FUN = scrape_single_url)

    # Detener el cluster
    parallel::stopCluster(cl)

    news_df$contenido_limpio <- unlist(results_list)
    message("Scrapeo en paralelo completado.")


  } else {
    # --- EJECUCION SECUENCIAL (con barra de progreso) ---
    message(paste("Iniciando scrapeo secuencial para", nrow(news_df), "articulos..."))
    pb <- utils::txtProgressBar(min = 0, max = nrow(news_df), style = 3)

    for (i in 1:nrow(news_df)) {
      if (!is.na(news_df$contenido_limpio[i]) && nzchar(news_df$contenido_limpio[i])) {
        utils::setTxtProgressBar(pb, i)
        next
      }

      # La logica de scrape_single_url se repite aqui para el modo secuencial
      current_url <- news_df$url[i]
      content_selector <- "div.col-lg-9 p.texto-nota"
      user_agent <- "datamedios R package scraper (https://github.com/exetrujillo/datamedios)"
      max_retries <- 3
      retry_delay <- 5

      response <- NULL
      for (attempt in 1:max_retries) {
        response <- tryCatch(
          httr::GET(current_url, httr::user_agent(user_agent), httr::timeout(30)),
          error = function(e) e
        )
        if (!inherits(response, "error") && httr::status_code(response) < 500) break
        Sys.sleep(retry_delay)
      }

      extracted_text <- if (inherits(response, "error")) {
        paste("REQUEST_ERROR:", response$message)
      } else if (httr::status_code(response) == 200) {
        page_html <- rvest::read_html(response)
        content_nodes <- rvest::html_nodes(page_html, content_selector)
        if (length(content_nodes) > 0) {
          stringr::str_squish(paste(rvest::html_text(content_nodes), collapse = "\n\n"))
        } else { "" }
      } else { paste("HTTP_ERROR:", httr::status_code(response)) }

      news_df$contenido_limpio[i] <- extracted_text
      utils::setTxtProgressBar(pb, i)
      Sys.sleep(0.3) # Pequeña pausa para no ser agresivo
    }
    close(pb)
    message("\nScrapeo secuencial completado.")
  }

  return(news_df)
}

#' Generar una tabla estilizada con las palabras mas frecuentes
#'
#' Esta funcion procesa la columna 'contenido_limpio' de un dataframe, tokeniza el texto,
#' cuenta la frecuencia de cada palabra y genera una tabla con las palabras mas frecuentes.
#'
#' @param datos Data frame que contiene la columna 'contenido_limpio'.
#' @param max_words Numero maximo de palabras que se mostraran en la tabla.
#' @param stop_words Vector opcional de palabras que se deben excluir del conteo.
#' @return Una tabla con las palabras mas frecuentes.
#' @examples
#' datos <- data.frame(
#'   contenido_limpio = c(
#'     "La ministra de Defensa Maya Fernandez enfrenta cuestionamientos
#'     el presidente Gabriel Boric solicita transparencia en los procesos.
#'     Renovacion Nacional pide la renuncia de Maya Fernandez debido a la polemica.
#'     La transparencia es fundamental en la politica y la gestion publica"
#'   ),
#'   stringsAsFactors = FALSE
#' )
#'
#' # Probar la funcion con el dataframe de ejemplo
#' tabla_frecuencia_palabras(datos, max_words = 5, stop_words = c())
#' @export

tabla_frecuencia_palabras <- function(datos, max_words, stop_words = NULL) {
  # Validar que 'datos' sea un data frame
  if (!is.data.frame(datos)) {
    stop("'datos' debe ser un data frame.")
  }
  # Validar que 'contenido_limpio' exista en los datos
  if (!"contenido_limpio" %in% colnames(datos)) {
    stop("'datos' debe contener una columna llamada 'contenido_limpio'.")
  }
  # Validar que 'max_words' sea numerico y este definido
  if (missing(max_words) || !is.numeric(max_words)) {
    stop("'max_words' debe ser un numero.")
  }
  # Validar el argumento 'stop_words'
  if (!is.null(stop_words) && !is.character(stop_words)) {
    stop("'stop_words' debe ser un vector de palabras.")
  }

  # Generar los tokens (separar palabras)
  words <- datos %>%
    tidytext::unnest_tokens(word, contenido_limpio)

  # Filtrar stop words si se proporcionan
  if (!is.null(stop_words)) {
    stop_words_df <- data.frame(word = stop_words)
    words <- words %>%
      dplyr::anti_join(stop_words_df, by = "word")
  }

  # Calcular frecuencia de palabras
  word_counts <- words %>%
    dplyr::count(word, sort = TRUE)

  # Filtrar las palabras mas frecuentes
  word_counts_filtered <- word_counts %>%
    dplyr::slice_max(n, n = max_words, with_ties = FALSE)

  # Generar la tabla estilizada
  word_counts_filtered %>%
    DT::datatable(
      options = list(
        pageLength = 10,            # Numero de filas por pagina
        autoWidth = TRUE,           # Ajusta el ancho de las columnas automaticamente
        columnDefs = list(
          list(className = 'dt-center', targets = "_all")  # Centra el contenido de las celdas
        )
      ),
      colnames = c("Palabra", "Frecuencia"),  # Nombres de columnas
      class = "display"  # Estilo de tabla
    )
}

test_that("Error cuando el input no es un data frame", {
  expect_error(
    tabla_frecuencia_palabras(list(a = 1, b = 2), max_words = 5),
    "'datos' debe ser un data frame."
  )
})

test_that("Error cuando falta la columna 'contenido_limpio'", {
  expect_error(
    tabla_frecuencia_palabras(data.frame(otra_columna = c("Texto")), max_words = 5),
    "'datos' debe contener una columna llamada 'contenido_limpio'."
  )
})

test_that("Error cuando 'max_words' no es numérico", {
  datos <- data.frame(contenido_limpio = c("Texto relevante."))
  expect_error(
    tabla_frecuencia_palabras(datos, max_words = "cinco"),
    "'max_words' debe ser un numero."
  )
})

test_that("Error cuando 'stop_words' no es un vector de caracteres", {
  datos <- data.frame(contenido_limpio = c("Texto relevante."))
  expect_error(
    tabla_frecuencia_palabras(datos, max_words = 5, stop_words = 123),
    "'stop_words' debe ser un vector de palabras."
  )
})

test_that("Tabla de palabras frecuentes funciona correctamente con datos válidos", {
  # Datos de prueba usando la función `extraer_noticias_max_res`
  datos <- extraer_noticias_max_res("boric", max_results = 10, subir_a_bd = FALSE)
  datos <- limpieza_notas(datos)

  # Generar la tabla
  tabla <- tabla_frecuencia_palabras(datos, max_words = 5)

  # Verificar que la tabla generada es un objeto de clase datatable
  expect_s3_class(tabla, "datatables")

  # Verificar que las palabras y frecuencias sean correctas
  palabras_frecuentes <- datos %>%
    tidytext::unnest_tokens(word, contenido_limpio) %>%
    dplyr::count(word, sort = TRUE) %>%
    dplyr::slice_max(n, n = 5)

  # Comparar las palabras en la tabla generada
  expect_equal(
    palabras_frecuentes$word,
    tabla$x$data[[2]], # Primera columna de la tabla generada
    info = "Las palabras frecuentes no coinciden."
  )
})

test_that("Stop words son excluidas correctamente", {
  datos <- data.frame(contenido_limpio = c("Esto es un ejemplo. Esto es una prueba."))

  # Generar la tabla excluyendo palabras comunes
  tabla <- tabla_frecuencia_palabras(datos, max_words = 3, stop_words = c("esto", "es", "un", "una"))

  # Verificar que las palabras stop fueron excluidas
  palabras_en_tabla <- tabla$x$data[[1]] # Primera columna de la tabla generada
  expect_false(any(palabras_en_tabla %in% c("esto", "es", "un", "una")))
})

#' Extraccion de noticias de medios chilenos por rango de fechas
#'
#' Esta funcion permite realizar una extraccion automatizada de noticias de
#' BioBio o Los medios de Emol, utilizando un rango de fechas.
#'
#' @param search_query Una frase de busqueda (opcional). Si la fuente es 'ciper', puede ser NULL.
#' @param fecha_inicio Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @param fecha_fin Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @param subir_a_bd por defecto TRUE, FALSE para test y cosas por el estilo (opcional).
#' @param fuentes es un string con las fuentes a extraer. Puede ser bbcl o las de emol.
#' @return Un dataframe con las noticias extraidas.
#' @examples
#' \dontrun{
#' noticias <- extraer_noticias_fecha("delincuencia", "2025-04-25",
#' "2025-04-28", subir_a_bd = FALSE, fuentes="bbcl")
#' }
#' @export

extraer_noticias_fecha <- function(search_query = NULL, fecha_inicio, fecha_fin, subir_a_bd = TRUE, fuentes="todas") {
  # Validamos los parametros
  if (is.null(search_query) && fuentes != "ciper") {
    stop("Debe proporcionar una frase de busqueda si la fuente no es exclusivamente 'ciper'.")
  }
  if (!is.null(search_query) && !is.character(search_query)) {
    stop("La frase de busqueda debe ser texto.")
  }
  if (missing(fecha_inicio) || missing(fecha_fin) || !lubridate::is.Date(lubridate::ymd(fecha_inicio)) || !lubridate::is.Date(lubridate::ymd(fecha_fin))) {
    stop("Debe proporcionar fechas de inicio y fin validas en formato 'YYYY-MM-DD'.")
  }
  if (lubridate::ymd(fecha_inicio) > lubridate::ymd(fecha_fin)) {
    stop("La fecha de inicio debe ser anterior o igual a la fecha de fin.")
  }

  ##############################################################################
  # Inicializamos variables y objetos
  patronFuentes = ""
  all_data <- data.frame(
    ID = character(),
    titulo = character(),
    contenido = character(),
    contenido_limpio = character(),
    url = character(),
    url_imagen = character(),
    autor = character(),
    fecha = character(),
    temas = list(),
    resumen = character(),
    search_query = character(),
    medio = character(),
    stringsAsFactors = FALSE
  )

  # Indice de fuentes
  if(fuentes=="todas"){
    patronFuentes <- "bbcl, emol-todas, ciper"
  } else {
    patronFuentes <- fuentes
  }

  fuentesParseadas <- parserFuentes(patronFuentes)

  message(paste0("\nFuentes parseadas: ", fuentesParseadas))
  ##############################################################################
  # CONJUNTOS DE FUENTES

  # emol
  fuentes_emol <- c("emol", "mediosregionales", "guioteca")

  ##############################################################################
  # SELECTOR DE EJECUCIONES

  #### BBCL ####
  if ("bbcl" %in% fuentesParseadas) {
    # Ejecutar la funcion para bbcl
    data_bbcl <- extraer_noticias_fecha_bbcl(search_query, fecha_inicio = fecha_inicio, fecha_fin = fecha_fin)
    if (!is.null(data_bbcl) && nrow(data_bbcl) > 0) {
      all_data <- rbind(all_data, data_bbcl)
    }
  }

  #### EMOL ####
  if ("emol-todas" %in% fuentesParseadas) {
    # Ejecutar la funcion para cada fuente de emol por separado
    for (fuente in fuentes_emol) {
      data_emol <- extraer_noticias_fecha_emol(search_query, fecha_inicio, fecha_fin, fuente = fuente)
      if (!is.null(data_emol) && nrow(data_emol) > 0) {
        all_data <- rbind(all_data, data_emol)
      }
    }
  } else {
    # Filtrar las fuentes de emol seleccionadas
    fuentes_emol_seleccionadas <- intersect(fuentesParseadas, fuentes_emol)
    if (length(fuentes_emol_seleccionadas) > 0) {
      # Ejecutar la funcion para cada fuente seleccionada individualmente
      for (fuente in fuentes_emol_seleccionadas) {
        data_emol <- extraer_noticias_fecha_emol(search_query, fecha_inicio, fecha_fin, fuente = fuente)
        if (!is.null(data_emol) && nrow(data_emol) > 0) {
          all_data <- rbind(all_data, data_emol)
        }
      }
    }
  }

  #### Ciper ####
  if ("ciper" %in% fuentesParseadas) {
    # Ejecutar la funcion para ciper
    data_ciper <- extraer_noticias_fecha_ciper(search_query, fecha_inicio = fecha_inicio, fecha_fin = fecha_fin)
    if (!is.null(data_ciper) && nrow(data_ciper) > 0) {
      all_data <- rbind(all_data, data_ciper)
    }
  }

  #### ACA IREMOS AGREGANDO MAS FUENTES CON SUS FUNCIONES ESPECIFICAS ####

  ##############################################################################

  # Subimos a la base de datos en caso de que el parametro subir_a_db es TRUE
  if (subir_a_bd && nrow(all_data) > 0) {
    tryCatch({
      # Llamamos a la funcion que sube los datos si subir_a_bd es TRUE
      agregar_datos_unicos(all_data)
    }, error = function(e) {
      message("Ocurrio un error al intentar agregar los datos a la base de datos: ", e$message)
    })
  }

  ##############################################################################

  return(all_data)
}

testthat::test_that("Se recibe un objeto de clase data frame, con 388 resultados y las columnas esperadas", {

  resultado <- extraer_noticias_fecha("boric", "2023-01-01", "2023-02-01", subir_a_bd = FALSE)

  #Verificamos que es data frame
  expect_s3_class(resultado, "data.frame")

  #Verificamos la cantidad de resultados (Revisar cada vez que agreguemos medios)
  expect_equal(nrow(resultado), 388) # No debería cambiar la cantidad de notas ya publicadas de esa fecha

  #Verificamos la existencia de las columnas deseadas
  expect_true("ID" %in% colnames(resultado))
  expect_true("titulo" %in% colnames(resultado))
  expect_true("contenido" %in% colnames(resultado))
  expect_true("contenido_limpio" %in% colnames(resultado))
  expect_true("url" %in% colnames(resultado))
  expect_true("url_imagen" %in% colnames(resultado))
  expect_true("autor" %in% colnames(resultado))
  expect_true("fecha" %in% colnames(resultado))
  expect_true("resumen" %in% colnames(resultado))
  expect_true("search_query" %in% colnames(resultado))
  expect_true("medio" %in% colnames(resultado))
  expect_true("temas" %in% colnames(resultado))

})

#' Extrae noticias de Ciper Chile por rango de fechas
#'
#' Esta funcion se conecta a la API de Ciper para descargar noticias dentro de un rango de fechas.
#'
#' @param search_query El termino de busqueda (opcional).
#' @param fecha_inicio Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @param fecha_fin Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @return Un dataframe con las noticias extraidas, estandarizado al formato de datamedios.
#' @export
#' @import dplyr
#' @importFrom magrittr %>%
extraer_noticias_fecha_ciper <- function(search_query = NULL, fecha_inicio, fecha_fin) {
  base_url <- "https://www.ciperchile.cl"
  api_endpoint <- paste0(base_url, "/wp-json/wp/v2/posts")
  user_agent <- "datamedios R package scraper (https://github.com/exetrujillo/datamedios)"

  all_posts <- list()
  current_page <- 1
  total_pages <- Inf
  max_retries <- 3
  retry_delay <- 5

  # Convertir fechas a formato ISO8601
  after_date <- paste0(fecha_inicio, "T00:00:00")
  before_date <- paste0(fecha_fin, "T23:59:59")

  message(paste("Iniciando extraccion desde Ciper para la busqueda:", search_query, "entre", fecha_inicio, "y", fecha_fin))

  pb <- NULL

  while (current_page <= total_pages) {
    query_params <- list(
      search = if (!is.null(search_query) && nzchar(search_query)) search_query else NULL,
      after = after_date,
      before = before_date,
      per_page = 100,
      page = current_page
    )

    response <- NULL
    for (attempt in 1:max_retries) {
      response <- httr::GET(url = api_endpoint, query = query_params, httr::user_agent(user_agent), httr::timeout(60))
      if (httr::status_code(response) < 500) {
        break
      }
      message(paste("\nError 500 en el servidor, reintentando en", retry_delay, "segundos..."))
      Sys.sleep(retry_delay)
    }

    if (httr::status_code(response) != 200) {
      warning(paste("\nError en la solicitud a la API de Ciper. Codigo:", httr::status_code(response)))
      break
    }

    if (current_page == 1) {
      headers <- httr::headers(response)
      total_posts <- as.integer(headers$`x-wp-total`)
      total_pages <- as.integer(headers$`x-wp-totalpages`)
      message(paste("Total de resultados encontrados en Ciper:", total_posts))
    }
    
    if (current_page == 1 && total_pages > 0) {
      pb <- utils::txtProgressBar(min = 0, max = total_pages, style = 3)
    }

    posts_data <- jsonlite::fromJSON(httr::content(response, "text", encoding = "UTF-8"), flatten = TRUE)

    if (length(posts_data) == 0 || NROW(posts_data) == 0) {
      message("No hay noticias en el rango de fechas seleccionado.")
      break
    }

    all_posts <- c(all_posts, split(posts_data, seq(nrow(posts_data))))

    if (!is.null(pb)) {
        utils::setTxtProgressBar(pb, current_page)
    }

    current_page <- current_page + 1
    Sys.sleep(0.5)
  }
  
  if (!is.null(pb)) close(pb)

  if (length(all_posts) == 0) {
    return(data.frame())
  }

  final_df <- dplyr::bind_rows(all_posts)

  column_map <- c(
    "id" = "ID", "link" = "url", "date" = "fecha", "title.rendered" = "titulo",
    "content.rendered" = "contenido", "excerpt.rendered" = "resumen"
  )
  existing_cols <- names(column_map)[names(column_map) %in% names(final_df)]

  output_df <- final_df %>%
    dplyr::select(dplyr::all_of(existing_cols)) %>%
    dplyr::rename_with(~ column_map[.], .cols = dplyr::all_of(existing_cols))

  output_df$search_query <- if (!is.null(search_query)) tolower(search_query) else NA_character_
  output_df$medio <- "ciper"
  output_df$contenido_limpio <- NA_character_
  output_df$url_imagen <- NA_character_
  output_df$autor <- NA_character_
  output_df$temas <- I(lapply(1:nrow(output_df), function(x) c()))

  output_df <- scrape_body_ciper(output_df)

  output_df$contenido <- output_df$contenido_limpio

  output_df <- output_df %>% 
    dplyr::mutate(ID = paste0(ID, "-c"))

  return(output_df)
}

#' Extraccion de noticias de medios chilenos por cantidad maxima de resultados
#'
#' Esta funcion permite realizar una extraccion automatizada de noticias
#' de BioBio y fuentes de El Mercurio.
#'
#' Es importante mencionar que si tiene mas de una fuente seleccionada, la
#' cantidad maxima de resultados se aplicara para cada una de las fuentes, es
#' decir, si pones max_results = 10 y tienes fuentes = "emol,guioteca,bbcl"
#' tendras como maximo 30 resultados.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param max_results Numero maximo de resultados a extraer (opcional, por defecto todos).
#' @param subir_a_bd por defecto TRUE, FALSE para test y cosas por el estilo (opcional).
#' @param fuentes por defecto marca todas las fuentes, pero se puede elegir una o varias de las disponibles en el README. (opcional)
#' @return Un dataframe con las noticias extraidas.
#' @examples
#' \dontrun{
#' noticias <- extraer_noticias_max_res("inteligencia artificial",
#' max_results = 20, fuentes="bbcl, emol", subir_a_bd = FALSE)
#' }
#' @export
extraer_noticias_max_res <- function(search_query = NULL, max_results = NULL, subir_a_bd = TRUE, fuentes = "todas") {
  # Validamos los parametros
  if (is.null(search_query) && fuentes != "ciper") {
    stop("Debe proporcionar una frase de busqueda si la fuente no es exclusivamente 'ciper'.")
  }
  if (!is.null(search_query) && !is.character(search_query)) {
    stop("La frase de busqueda debe ser texto.")
  }
  if (!is.null(max_results) && (!is.numeric(max_results) || max_results <= 0)) {
    stop("max_results debe ser un numero entero positivo o NULL.")
  }

  ##############################################################################

  # Inicializamos variables y objetos
  patronFuentes = ""
  all_data <- data.frame(
    ID = character(),
    titulo = character(),
    contenido = character(),
    contenido_limpio = character(),
    url = character(),
    url_imagen = character(),
    autor = character(),
    fecha = character(),
    temas = list(),
    resumen = character(),
    search_query = character(),
    medio = character(),
    stringsAsFactors = FALSE
  )

  # Indice de fuentes
  if(fuentes=="todas"){
    patronFuentes <- "bbcl, emol-todas, ciper"
  } else {
    patronFuentes <- fuentes
  }

  fuentesParseadas <- parserFuentes(patronFuentes)
  message(paste0("Fuentes parseadas: ", fuentesParseadas))

  ##############################################################################
  # CONJUNTOS DE FUENTES

  # emol
  fuentes_emol <- c("emol", "mediosregionales", "guioteca")

  ##############################################################################
  # SELECTOR DE EJECUCIONES

  #### BBCL ####
  if ("bbcl" %in% fuentesParseadas) {
    # Ejecutar la funcion para bbcl
    data_bbcl <- extraer_noticias_max_res_bbcl(search_query, max_results = max_results)
    if (!is.null(data_bbcl) && nrow(data_bbcl) > 0) {
      all_data <- rbind(all_data, data_bbcl)
    }
  }

  #### EMOL ####
  if ("emol-todas" %in% fuentesParseadas) {
    # Ejecutar la funcion para cada fuente de emol por separado
    for (fuente in fuentes_emol) {
      data_emol <- extraer_noticias_max_res_emol(search_query, max_results = max_results, fuente = fuente)
      if (!is.null(data_emol) && nrow(data_emol) > 0) {
        all_data <- rbind(all_data, data_emol)
      }
    }
  } else {
    # Filtrar las fuentes de emol seleccionadas
    fuentes_emol_seleccionadas <- intersect(fuentesParseadas, fuentes_emol)
    if (length(fuentes_emol_seleccionadas) > 0) {
      # Ejecutar la funcion para cada fuente seleccionada individualmente
      for (fuente in fuentes_emol_seleccionadas) {
        data_emol <- extraer_noticias_max_res_emol(search_query, max_results = max_results, fuente = fuente)
        if (!is.null(data_emol) && nrow(data_emol) > 0) {
          all_data <- rbind(all_data, data_emol)
        }
      }
    }
  }

  #### Ciper ####
  if ("ciper" %in% fuentesParseadas) {
    # Ejecutar la funcion para ciper
    data_ciper <- extraer_noticias_max_res_ciper(search_query, max_results = max_results)
    if (!is.null(data_ciper) && nrow(data_ciper) > 0) {
      all_data <- rbind(all_data, data_ciper)
    }
  }

  ##############################################################################


  ##############################################################################

  # Subimos a la base de datos en caso de que el parametro subir_a_db es TRUE
  if (subir_a_bd) {
    tryCatch({
      # Llamamos a la funcion que sube los datos si subir_a_bd es TRUE
      agregar_datos_unicos(all_data)
    }, error = function(e) {
      message("Ocurrio un error al intentar agregar los datos a la base de datos: ", e$message)
    })
  }

  message(paste0("Noticias hasta la fecha: ", nrow(all_data)))

  return(all_data)
}

# Test para validar errores por entradas incorrectas
test_that("Error cuando el input no es un data.frame", {
  expect_error(
    datamedios::limpieza_notas(list(a = 1, b = 2)),
    "El argumento 'datos' debe ser un data frame."
  )
})

test_that("Error cuando falta la columna 'contenido'", {
  expect_error(
    datamedios::limpieza_notas(data.frame(otra_columna = c("Texto"))),
    "El data frame debe contener una columna llamada 'contenido'."
  )
})

test_that("Errores cuando 'sinonimos' no es un vector de caracteres", {
  datos <- data.frame(contenido = c("Texto de prueba"), stringsAsFactors = FALSE)
  expect_error(
    datamedios::limpieza_notas(datos, sinonimos = 123),
    "'sinonimos' debe ser un vector de palabras."
  )
})

# Test para verificar filtrado por sinónimos
test_that("Se realiza el filtrado correctamente considerando los sinónimos", {
  # Datos generados con la función del paquete
  datos <- datamedios::extraer_noticias_max_res("boric", max_results = 100, subir_a_bd = FALSE)
  resultado <- datamedios::limpieza_notas(datos, sinonimos = c("presidente"))

  # Verificamos que solo queden filas relevantes
  expect_true(
    any(grepl("presidente", resultado$contenido) & !grepl("boric", resultado$contenido)),
    info = "No se encontró ninguna fila donde aparezca 'presidente' pero no 'boric'."
  )
})

# Test para limpiar datos extraídos con la función de tu paquete
test_that("La función limpia el texto de forma correcta en datos reales", {
  # Datos generados con la función del paquete
  datos <- datamedios::extraer_noticias_max_res("boric", max_results = 100, subir_a_bd = FALSE)

  # Ejecución de la función de limpieza
  resultado <- datamedios::limpieza_notas(datos)

  # Verificamos que no quede HTML en una fila aleatoria
  indice_aleatorio <- sample(1:nrow(resultado), 1)

  es_html <- tryCatch({
    xml2::read_html(resultado$contenido_limpio[indice_aleatorio])
    TRUE
  }, error = function(e) {
    print(paste("Error al procesar HTML en la fila:", indice_aleatorio))
    print(resultado$contenido_limpio[indice_aleatorio])
    print(paste("Mensaje de error:", e$message))
    FALSE
  })

  expect_false(
    es_html,
    info = paste("HTML no fue eliminado correctamente en la fila", indice_aleatorio)
  )
})

# Test final para verificar estructura, limpieza y columnas esperadas
test_that("Resultados finales tienen estructura correcta y columnas esperadas", {
  # Extracción inicial
  datos <- datamedios::extraer_noticias_max_res("boric", max_results = 10, subir_a_bd = FALSE)
  resultado <- datamedios::limpieza_notas(datos, c("presidente"))

  # Verificamos que el resultado es un data.frame
  expect_s3_class(resultado, "data.frame")

  # Verificamos que las columnas deseadas existen
  columnas_esperadas <- c(
    "ID", "titulo", "contenido", "contenido_limpio", "url", "url_imagen",
    "autor", "fecha", "resumen", "search_query", "medio",
    "temas"
  )
  for (columna in columnas_esperadas) {
    expect_true(
      columna %in% colnames(resultado),
      info = paste("La columna", columna, "no está en el resultado.")
    )
  }

  # Verificamos que no quede HTML en una fila aleatoria
  indice_aleatorio <- sample(1:nrow(resultado), 1)

  es_html <- tryCatch({
    xml2::read_html(resultado$contenido_limpio[indice_aleatorio])
    TRUE
  }, error = function(e) {
    print(paste("Error al procesar HTML en la fila:", indice_aleatorio))
    print(resultado$contenido_limpio[indice_aleatorio])
    print(paste("Mensaje de error:", e$message))
    FALSE
  })

  expect_false(
    es_html,
    info = paste("HTML no fue eliminado correctamente en la fila", indice_aleatorio)
  )
})

testthat::test_that("Se recibe un objeto de clase data frame, con 201 resultados y las columnas esperadas", {
  #NOTA: no todos los medios tienen 201 resultados, además de que la función extrae 201 por medio
  resultado <- extraer_noticias_max_res("boric", max_results = 201, subir_a_bd = FALSE)

  #Verificamos que es data frame
  expect_s3_class(resultado, "data.frame")

  #Verificamos la cantidad de resultados
  expect_lt(nrow(resultado), 201*5)

  #Verificamos la existencia de las columnas deseadas
  expect_true("ID" %in% colnames(resultado))
  expect_true("titulo" %in% colnames(resultado))
  expect_true("contenido" %in% colnames(resultado))
  expect_true("contenido_limpio" %in% colnames(resultado))
  expect_true("url" %in% colnames(resultado))
  expect_true("url_imagen" %in% colnames(resultado))
  expect_true("autor" %in% colnames(resultado))
  expect_true("fecha" %in% colnames(resultado))
  expect_true("resumen" %in% colnames(resultado))
  expect_true("search_query" %in% colnames(resultado))
  expect_true("medio" %in% colnames(resultado))
  expect_true("temas" %in% colnames(resultado))
})

test_that("La funcion procesa correctamente los datos sin sinonimos", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c(
      "<html><p>La inteligencia artificial está cambiando el mundo.</p></html>",
      "<html><p>No hay menciones aquí.</p></html>"
    ),
    stringsAsFactors = FALSE
  )
  resultado <- extraccion_parrafos(datos)
  expect_true("parrafos_filtrados" %in% colnames(resultado))
  expect_equal(resultado$parrafos_filtrados[[1]], "La inteligencia artificial está cambiando el mundo.")
  expect_equal(resultado$parrafos_filtrados[[2]], character(0))
})

test_that("La funcion procesa correctamente los datos con sinonimos", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c(
      "<html><p>La IA esta en todas partes.</p><p>La inteligencia artificial esta en auge.</p></html>",
      "<html><p>No hay menciones aqui.</p></html>"
    ),
    stringsAsFactors = FALSE
  )
  sinonimos <- c("IA", "AI")
  resultado <- extraccion_parrafos(datos, sinonimos)
  expect_equal(resultado$parrafos_filtrados[[1]], c("La IA esta en todas partes.", "La inteligencia artificial esta en auge."))
  expect_equal(resultado$parrafos_filtrados[[2]], character(0))
})

test_that("La función lanza un error si falta la columna 'contenido'", {
  datos <- data.frame(search_query = "inteligencia artificial")
  expect_error(extraccion_parrafos(datos), "El data frame debe contener la columna 'contenido'.")
})

test_that("La función lanza un error si 'datos' no es un data frame", {
  datos <- list(search_query = "inteligencia artificial", contenido = "Contenido")
  expect_error(extraccion_parrafos(datos), "'datos' debe ser un data frame.")
})

test_that("La función maneja contenido HTML mal formado", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c("<html><p>Texto válido</p>", "Contenido no HTML"),
    stringsAsFactors = FALSE
  )
  resultado <- extraccion_parrafos(datos)
  expect_true(all(is.na(resultado$parrafos_filtrados[[2]])))
})

test_that("La función maneja correctamente una columna 'contenido' vacía", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c("", ""),
    stringsAsFactors = FALSE
  )
  resultado <- extraccion_parrafos(datos)
  expect_true(all(sapply(resultado$parrafos_filtrados, is.na)))
})

test_that("La función procesa correctamente cuando 'sinonimos' está vacío", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c("<html><p>Texto sobre inteligencia artificial.</p></html>"),
    stringsAsFactors = FALSE
  )
  resultado <- extraccion_parrafos(datos, sinonimos = NULL)
  expect_equal(resultado$parrafos_filtrados[[1]], "Texto sobre inteligencia artificial.")
})

test_that("La función procesa textos largos con párrafos mixtos", {
  datos <- data.frame(
    search_query = "inteligencia artificial",
    contenido = c(
      "<html><p>La IA es una revolución.</p><p>Este es un párrafo irrelevante.</p><p>La inteligencia artificial cambiará el mundo.</p></html>"
    ),
    stringsAsFactors = FALSE
  )
  sinonimos <- c("IA", "AI")
  resultado <- extraccion_parrafos(datos, sinonimos)
  expect_equal(
    resultado$parrafos_filtrados[[1]],
    c("La IA es una revolución.", "La inteligencia artificial cambiará el mundo.")
  )
})

test_that("La función elimina correctamente el HTML en los párrafos extraídos", {
  # Generar datos simulados usando extraer_noticias_max_res
  datos <- extraer_noticias_max_res("boric", max_results = 20, subir_a_bd = FALSE)

  # Sinónimos para la búsqueda
  sinonimos <- c("IA", "AI")

  # Procesar los datos con extraccion_parrafos
  resultado <- extraccion_parrafos(datos, sinonimos)

  # Comprobar que no exista HTML en los párrafos extraídos
  contiene_html <- any(grepl("<[^>]+>", unlist(resultado$parrafos_filtrados), perl = TRUE))

  # Evaluar el test
  expect_false(contiene_html, info = "Los párrafos procesados no deberían contener HTML.")
})

library(testthat)

# Prueba para la extraccion de noticias de Ciper
test_that("extraer_noticias_max_res_ciper funciona correctamente", {
  # Evitar hacer llamadas reales a la API en tests automaticos
  skip_on_cran()

  # Termino de busqueda comun para asegurar resultados
  search_query <- "derechos humanos"
  max_results <- NULL

  # Ejecutar la funcion
  noticias <- extraer_noticias_max_res(search_query, max_results = 3, fuentes = "ciper", subir_a_bd = FALSE)

  # 1. Verificar que el resultado es un dataframe
  expect_s3_class(noticias, "data.frame")

  # 2. Verificar que el dataframe no este vacio
  expect_true(nrow(noticias) > 0)

  # 3. Verificar que el numero de filas es el esperado
  expect_equal(nrow(noticias), max_results)

  # 4. Verificar que las columnas esenciales esten presentes
  columnas_esperadas <- c("ID", "titulo", "contenido", "contenido_limpio", "url", "fecha", "resumen", "search_query", "medio")
  expect_true(all(columnas_esperadas %in% names(noticias)))

  # 5. Verificar que el medio es 'ciper'
  expect_true(all(noticias$medio == "ciper"))

  # 6. Verificar que la columna de contenido limpio no este vacia
  expect_true(all(!is.na(noticias$contenido_limpio)))
  expect_true(all(nchar(noticias$contenido_limpio) > 0))

})

#' Agregar datos unicos a una tabla MySQL
#'
#' Esta funcion agrega datos a una tabla MySQL utilizando endpoints que esperan datos en formato JSON.
#'
#' @param data Un data frame con los datos a insertar.
#' @return No retorna ningun valor.
#' @export
#' @examples
#' \dontrun{
#' # Agregar datos unicos
#' noticias <- extraer_noticias_max_res("tesla", max_results=10, fuentes="bbcl", subir_a_bd = FALSE)
#' agregar_datos_unicos(noticias)
#' }

agregar_datos_unicos <- function(data) {
  # Validamos entrada
  if (!is.data.frame(data)) {
    stop("El argumento 'data' debe ser un data frame")
  }
  if (nrow(data) == 0 || ncol(data) == 0) {
    stop("El data frame no puede estar vacio")
  }
  if (!"search_query" %in% colnames(data)) {
    stop("El data frame debe contener una columna llamada 'search_query'.")
  }

  # Eliminaciones necesarias
  data$parrafos_filtrados <- NULL # Si es que el dataframe viene con esta columna

  # Convertimos data a lista
  data_list <- as.list(data)

  # URLs de las APIs
  url1 <- "http://librosycodigo.ddns.net:3124/write_news"
  url2 <- "http://librosycodigo.ddns.net:3124/write_search_query"

  # Funcion para enviar datos a la API
  enviar_datos <- function(url, data) {
    if(url == url1){
      mensaje <- "endpoint de noticias"
    } else if (url==url2){
      mensaje <- "endpoint de queries"
    }

    response <- httr::POST(url, body = data, encode = "json")
    if (httr::http_status(response)$category != "Success") {
      stop(paste("Error al enviar datos a", mensaje, "- Codigo:", response$status_code))
    }
    return(response)
  }

  # Enviamos datos a write_news
  enviar_datos(url1, data_list)

  # Enviamos datos a write_search_queries
  invisible(enviar_datos(url2, data_list))

  #message("Datos agregados exitosamente.")
}

#' Extraer parrafos de una columna de texto
#'
#' Esta funcion procesa una columna de texto en un dataframe y extrae los parrafos que coinciden con los sinonimos proporcionados.
#'
#' @param datos Data frame que contiene los datos de entrada con la columna "contenido".
#' @param sinonimos Vector de sinonimos que se incluiran en la busqueda.
#' @return Data frame con una columna adicional 'parrafos_filtrados' que contiene los parrafos extraidos como listas.
#' @examples
#' \donttest{
#' datos <- extraer_noticias_max_res("inteligencia artificial", max_results = 140, subir_a_bd = FALSE)
#' datos <- extraccion_parrafos(datos, sinonimos = c("IA", "AI"))
#' }
#' @export

extraccion_parrafos <- function(datos, sinonimos = c()) {
  # Verificar que los datos sean un data frame y que contengan la columna 'contenido'
  if (!is.data.frame(datos)) stop("'datos' debe ser un data frame.")
  if (!"contenido" %in% colnames(datos)) stop("El data frame debe contener la columna 'contenido'.")

  if (!is.null(sinonimos)){
    pattern <- paste0("(?i)\\b(", datos$search_query[[1]], "|", paste(sinonimos, collapse = "|"), ")\\b")
  } else {
    pattern <- paste0("(?i)\\b(", datos$search_query[[1]], ")\\b")
  }

  # Procesar cada contenido de 'contenido' para extraer parrafos filtrados
  datos <- datos %>%
    dplyr::mutate(
      parrafos_filtrados = purrr::map(contenido, ~ {
        # Manejo de errores para lectura HTML
        nodo_html <- tryCatch(rvest::read_html(.x), error = function(e) return(NA))

        if (!is.na(nodo_html)) {
          # Extraer parrafos del HTML
          parrafos <- nodo_html %>% rvest::html_elements("p") %>% rvest::html_text2()

          # Filtrar parrafos que coincidan con los sinonimos
          parrafos[grepl(pattern, parrafos)]
        } else {
          return(NA) # En caso de error
        }
      })
    )

  return(datos)
}

#' Grafico de notas por periodo (interactivo)
#'
#' Esta funcion genera un grafico interactivo que muestra la cantidad de publicaciones
#' agrupadas por dia o por mes, con opcion de tema dark o light.
#'
#' @param datos Data frame con los datos procesados, que debe incluir la columna `fecha` en formato YYYY-MM-DD.
#' @param titulo Texto que aparecera en el titulo del grafico.
#' @param fecha_inicio Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @param fecha_fin Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @param agrupar_por Cadena de texto que especifica el periodo de agrupacion.
#'   Valores validos son `"day"` (por defecto) o `"month"`.
#' @param tema Tema del grafico. Valores validos son `"light"` (por defecto) o `"dark"`.
#' @return Un grafico plotly interactivo que muestra la cantidad de publicaciones por el periodo seleccionado.
#' @examples
#' \dontrun{
#' # Ejemplo con tema dark, agrupando por mes
#' datos <- extraer_noticias_fecha("cambio climatico", "2024-01-01","2025-01-01", subir_a_bd = FALSE)
#' grafico_notas_fecha(datos, titulo = "Cambio Climatico (por mes)",
#'                     agrupar_por = "month", tema = "dark")
#'
#' # Ejemplo con tema light, agrupando por dia
#' grafico_notas_fecha(datos, titulo = "Cambio Climatico (por dia)",
#'                     fecha_inicio = "2024-01-01", fecha_fin = "2024-03-31",
#'                     tema = "light")
#' }
#'
#' @export
grafico_notas_fecha <- function(datos, titulo, fecha_inicio = NULL, fecha_fin = NULL,
                                agrupar_por = "day", tema = "light") {

  # Cargar librerias necesarias
  if (!requireNamespace("plotly", quietly = TRUE)) {
    stop("El paquete 'plotly' es necesario para esta funcion. instalalo con: install.packages('plotly')")
  }

  # --- 1. Validacion de Entradas ---

  # Validar que 'datos' sea un data frame
  if (!is.data.frame(datos)) {
    stop("'datos' debe ser un data frame.")
  }
  # Validar que 'fecha' exista en los datos
  if (!"fecha" %in% colnames(datos)) {
    stop("'datos' debe contener la columna 'fecha'.")
  }
  # Validar que 'titulo' sea texto
  if (!is.character(titulo) || length(titulo) != 1) {
    stop("'titulo' debe ser una unica cadena de texto (string).")
  }
  # Validar el parametro 'agrupar_por'
  if (!agrupar_por %in% c("month", "day")) {
    stop("'agrupar_por' debe ser 'month' o 'day'.")
  }
  # Validar el parametro 'tema'
  if (!tema %in% c("light", "dark")) {
    stop("'tema' debe ser 'light' o 'dark'.")
  }

  # --- 2. Configuracion de Colores por Tema ---

  if (tema == "dark") {
    colores <- list(
      fondo = "#1a1a1a",
      texto = "#ffffff",
      grilla = "#404040",
      linea_principal = "#00d4ff",
      puntos = "#ff6b35",
      suavizado = "#39ff14",
      barras = "#00d4ff"
    )
  } else { # tema == "light"
    colores <- list(
      fondo = "#ffffff",
      texto = "#333333",
      grilla = "#e0e0e0",
      linea_principal = "#1f77b4",
      puntos = "#d62728",
      suavizado = "#2ca02c",
      barras = "#1f77b4"
    )
  }

  # --- 3. Preparacion y Filtrado de Datos ---

  # Convertir 'fecha' a formato fecha y asegurar que no hay horas/minutos
  datos_proc <- datos %>%
    dplyr::mutate(fecha = as.Date(fecha, format = "%Y-%m-%d"))

  # Filtrar por rango de fechas si se especifican
  if (!is.null(fecha_inicio) || !is.null(fecha_fin)) {
    if (!is.null(fecha_inicio)) {
      fecha_inicio <- as.Date(fecha_inicio, format = "%Y-%m-%d")
    } else {
      fecha_inicio <- min(datos_proc$fecha, na.rm = TRUE)
    }
    if (!is.null(fecha_fin)) {
      fecha_fin <- as.Date(fecha_fin, format = "%Y-%m-%d")
    } else {
      fecha_fin <- max(datos_proc$fecha, na.rm = TRUE)
    }

    # Validar que las fechas sean validas
    if (is.na(fecha_inicio) || is.na(fecha_fin)) {
      stop("Las fechas de inicio/fin no son validas o no hay fechas en los datos.")
    }
    if (fecha_inicio > fecha_fin) {
      stop("'fecha_inicio' debe ser anterior o igual a 'fecha_fin'.")
    }

    # Filtrar datos por rango
    datos_filtrados <- datos_proc %>%
      dplyr::filter(fecha >= fecha_inicio & fecha <= fecha_fin)
  } else {
    # Usar todo el rango de fechas disponible
    datos_filtrados <- datos_proc
  }

  # Verificar si hay datos despues del filtrado
  if (nrow(datos_filtrados) == 0) {
    stop("No hay datos en el rango de fechas seleccionado.")
  }

  # --- 4. Agregacion de Datos Condicional ---

  if (agrupar_por == "month") {
    # Agrupar datos por mes y contar las publicaciones
    publicaciones_agrupadas <- datos_filtrados %>%
      dplyr::mutate(fecha_grupo = lubridate::floor_date(fecha, "month")) %>%
      dplyr::group_by(fecha_grupo) %>%
      dplyr::summarise(cantidad = dplyr::n(), .groups = "drop") %>%
      dplyr::arrange(fecha_grupo)

  } else { # agrupar_por == "day"
    # Agrupar datos por dia y contar las publicaciones
    publicaciones_agrupadas <- datos_filtrados %>%
      dplyr::group_by(fecha) %>%
      dplyr::rename(fecha_grupo = fecha) %>%
      dplyr::summarise(cantidad = dplyr::n(), .groups = "drop") %>%
      dplyr::arrange(fecha_grupo)
  }

  # --- 5. Generacion del Grafico Base con ggplot2 ---

  # Crear el grafico base
  grafico_base <- ggplot2::ggplot(publicaciones_agrupadas, ggplot2::aes(x = fecha_grupo, y = cantidad)) +
    ggplot2::labs(
      title = titulo,
      x = "Fecha",
      y = "Cantidad de Notas"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.background = ggplot2::element_rect(fill = colores$fondo, color = NA),
      panel.background = ggplot2::element_rect(fill = colores$fondo, color = NA),
      text = ggplot2::element_text(color = colores$texto),
      axis.text = ggplot2::element_text(color = colores$texto),
      axis.title = ggplot2::element_text(color = colores$texto),
      plot.title = ggplot2::element_text(color = colores$texto, hjust = 0.5, size = 14, face = "bold"),
      panel.grid.major = ggplot2::element_line(color = colores$grilla, size = 0.3),
      panel.grid.minor = ggplot2::element_line(color = colores$grilla, size = 0.1),
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
    )

  # Añadir las capas (geoms) segun el tipo de agrupacion
  if (agrupar_por == "month") {
    grafico_ggplot <- grafico_base +
      ggplot2::geom_line(color = colores$linea_principal, linewidth = 1.2) +
      ggplot2::geom_point(color = colores$puntos, size = 2.5) +
      ggplot2::geom_smooth(method = "loess", color = colores$suavizado, se = FALSE, linewidth = 1)
  } else { # agrupar_por == "day"
    grafico_ggplot <- grafico_base +
      ggplot2::geom_col(fill = colores$barras, color = colores$barras, width = 1, alpha = 0.8)
  }

  # --- 6. Conversion a Plotly interactivo ---

  # Convertir a plotly
  grafico_plotly <- plotly::ggplotly(grafico_ggplot, tooltip = c("x", "y")) %>%
    plotly::layout(
      plot_bgcolor = colores$fondo,
      paper_bgcolor = colores$fondo,
      font = list(color = colores$texto, family = "Arial", size = 12),
      title = list(
        text = titulo,
        font = list(color = colores$texto, size = 16, family = "Arial"),
        x = 0.5
      ),
      xaxis = list(
        title = list(text = "Fecha", font = list(color = colores$texto)),
        tickfont = list(color = colores$texto),
        gridcolor = colores$grilla,
        zerolinecolor = colores$grilla
      ),
      yaxis = list(
        title = list(text = "Cantidad de Notas", font = list(color = colores$texto)),
        tickfont = list(color = colores$texto),
        gridcolor = colores$grilla,
        zerolinecolor = colores$grilla
      ),
      hovermode = "x unified",
      showlegend = FALSE
    ) %>%
    plotly::config(
      displayModeBar = TRUE,
      modeBarButtonsToRemove = c("pan2d", "select2d", "lasso2d", "autoScale2d"),
      displaylogo = FALSE,
      toimageButtonOptions = list(
        format = "png",
        filename = paste0("grafico_", gsub(" ", "_", tolower(titulo))),
        height = 500,
        width = 800,
        scale = 2
      )
    )

  # --- 7. Personalizar Tooltips ---

  # Crear etiquetas personalizadas para el hover
  if (agrupar_por == "month") {
    publicaciones_agrupadas <- publicaciones_agrupadas %>%
      dplyr::mutate(
        fecha_texto = format(fecha_grupo, "%Y-%m"),
        hover_text = paste0("Fecha: ", fecha_texto, "<br>Cantidad: ", cantidad)
      )

    # Actualizar las trazas de plotly con texto personalizado
    for (i in 1:length(grafico_plotly$x$data)) {
      if (!is.null(grafico_plotly$x$data[[i]]$x)) {
        grafico_plotly$x$data[[i]]$text <- publicaciones_agrupadas$hover_text
        grafico_plotly$x$data[[i]]$hovertemplate <- "%{text}<extra></extra>"
      }
    }
  } else {
    publicaciones_agrupadas <- publicaciones_agrupadas %>%
      dplyr::mutate(
        fecha_texto = format(fecha_grupo, "%Y-%m-%d"),
        hover_text = paste0("Fecha: ", fecha_texto, "<br>Cantidad: ", cantidad)
      )

    # Actualizar las trazas de plotly con texto personalizado
    for (i in 1:length(grafico_plotly$x$data)) {
      if (!is.null(grafico_plotly$x$data[[i]]$x)) {
        grafico_plotly$x$data[[i]]$text <- publicaciones_agrupadas$hover_text
        grafico_plotly$x$data[[i]]$hovertemplate <- "%{text}<extra></extra>"
      }
    }
  }

  # --- 8. Devolver el Grafico interactivo ---
  return(grafico_plotly)
}

#' Funcion de nube de palabras
#'
#' Esta funcion permite realizar una nube de palabras con las palabras más frecuentes del corpus de noticias.
#'
#' @param datos data frame que incluye la columna contenido_limpio.
#' @param max_words Cantidad maxima de palabras que apareceran en la nube.
#' @param stop_words Definir las palabras que seran ignoradas en la visualizacion. Debe ser un vector de carácteres.
#' @return Una nube de palabras con las palabras mas frecuentes.
#' @examples
#' \dontrun{
#' datos <- extraer_noticias_fecha("Boric",
#' "2025-03-01",
#' "2025-04-01",
#' fuentes="bbcl",
#' subir_a_bd = FALSE)
#' datos_proc <- limpieza_notas(datos)
#' word_cloud(datos_proc, max_words = 50, stop_words = c("es", "la"))
#' }
#' @export

word_cloud <- function(datos, max_words, stop_words = NULL) {
  # Validar que 'datos' sea un data frame
  if (!is.data.frame(datos)) {
    stop("'datos' debe ser un data frame.")
  }

  # Validar que 'contenido_limpio' exista en los datos
  if (!"contenido_limpio" %in% colnames(datos)) {
    stop("'datos' debe contener una columna llamada 'contenido_limpio'.")
  }

  # Validar que 'max_words' sea numerico y este definido
  if (missing(max_words) || !is.numeric(max_words)) {
    stop("'max_words' debe ser un numero.")
  }

  # Validar el argumento 'stop_words'
  if (!is.null(stop_words) && !is.character(stop_words)) {
    stop("'stop_words' debe ser un vector de palabras.")
  }

  # Generar los tokens (separar palabras)
  words <- datos %>%
    tidytext::unnest_tokens(word, contenido_limpio)

  # Filtrar stop words si se proporcionan
  if (!is.null(stop_words)) {
    stop_words_df <- data.frame(word = stop_words)
    words <- words %>%
      dplyr::anti_join(stop_words_df, by = "word")
  }

  # Verificar si hay palabras suficientes para generar la nube
  if (nrow(words) == 0) {
    stop("No se encontraron palabras para generar la nube.")
  }

  # Calcular frecuencia de palabras
  word_counts <- words %>%
    dplyr::count(word, sort = TRUE)

  # Filtrar las palabras mas frecuentes
  word_counts_filtered <- word_counts %>%
    dplyr::slice_max(n, n = max_words)

  # Generar la nube de palabras
  wordcloud2::wordcloud2(
    data = word_counts_filtered,
    size = 0.3,              # Aumenta el tamano general de las palabras
    minSize = 0,             # Asegura que todas las palabras sean visibles
    gridSize = 1,            # Ajusta la densidad de palabras
    color = "random-dark",   # Colores para las palabras
    backgroundColor = "white", # Fondo blanco
    shape = "circle",        # Forma circular para compactar la nube
    ellipticity = 1          # Elimina la elipse y fuerza un formato mas centrado
  )
}

test_that("Error cuando el input no es un data frame", {
  expect_error(
    word_cloud(list(a = 1, b = 2), max_words = 50),
    "'datos' debe ser un data frame."
  )
})

test_that("Error cuando falta la columna 'contenido_limpio'", {
  expect_error(
    word_cloud(data.frame(otra_columna = c("Texto")), max_words = 50),
    "'datos' debe contener una columna llamada 'contenido_limpio'."
  )
})

test_that("Error cuando 'max_words' no es numérico", {
  datos <- data.frame(contenido_limpio = c("Texto relevante."))
  expect_error(
    word_cloud(datos, max_words = "cincuenta"),
    "'max_words' debe ser un numero."
  )
})

test_that("Error cuando 'stop_words' no es un vector de caracteres", {
  datos <- data.frame(contenido_limpio = c("Texto relevante."))
  expect_error(
    word_cloud(datos, max_words = 50, stop_words = 123),
    "'stop_words' debe ser un vector de palabras."
  )
})

test_that("La función genera una nube de palabras correctamente", {
  datos <- data.frame(
    contenido_limpio = c(
      "La inteligencia artificial está transformando el mundo.",
      "La transformación digital depende de la inteligencia artificial."
    )
  )

  # Generar la nube de palabras
  nube <- word_cloud(datos, max_words = 5)

  # Verificar que se genera un objeto de clase wordcloud2
  expect_s3_class(nube, "wordcloud2")

  # Verificar las palabras más frecuentes
  palabras_frecuentes <- datos %>%
    tidytext::unnest_tokens(word, contenido_limpio) %>%
    dplyr::count(word, sort = TRUE) %>%
    dplyr::slice_max(n, n = 5)

  # Asegurarse de que las palabras aparecen en la nube
  palabras_en_nube <- nube$x$word
  expect_true(all(palabras_frecuentes$word %in% palabras_en_nube))
})

test_that("Stop words son excluidas correctamente", {
  datos <- data.frame(contenido_limpio = c("Esto es un ejemplo. Esto es una prueba."))

  # Generar la nube de palabras excluyendo stop words
  nube <- word_cloud(datos, max_words = 3, stop_words = c("esto", "es", "un", "una"))

  # Verificar que las palabras stop no están en la nube
  palabras_en_nube <- nube$x$word
  expect_false(any(palabras_en_nube %in% c("esto", "es", "un", "una")))
})

test_that("La función maneja casos límite de datos correctamente", {
  datos <- data.frame(contenido_limpio = c(""))

  # Generar la nube con un texto vacío
  expect_error(
    word_cloud(datos, max_words = 5),
    "No se encontraron palabras para generar la nube."
  )

  # Generar la nube con un único dato
  datos <- data.frame(contenido_limpio = c("inteligencia artificial"))
  nube <- word_cloud(datos, max_words = 5)

  # Verificar que la palabra aparece en la nube
  palabras_en_nube <- nube$x$word
  expect_true("inteligencia" %in% palabras_en_nube)
  expect_true("artificial" %in% palabras_en_nube)
})

test_that("'max_words' limita correctamente la cantidad de palabras en la nube", {
  datos <- data.frame(
    contenido_limpio = c(
      "La inteligencia artificial está transformando el mundo.",
      "La transformación digital depende de la inteligencia artificial.",
      "El futuro está marcado por la inteligencia artificial y la digitalización."
    )
  )

  # Generar la nube con max_words = 3
  nube <- word_cloud(datos, max_words = 3)

  # Obtener las palabras en la nube
  palabras_en_nube <- nube$x$word

  # Verificar que el número de palabras no excede max_words
  expect_lte(length(palabras_en_nube), 3)
})

#' Grafico de notas por mes
#'
#' Esta funcion genera un grafico de linea que muestra la cantidad de publicaciones agrupadas por mes.
#'
#' @param datos Data frame con los datos procesados, que debe incluir la columna `fecha` en formato YYYY-MM-DD.
#' @param titulo Texto que aparecera en el titulo del grafico.
#' @param fecha_inicio Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @param fecha_fin Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).
#' @return Un grafico ggplot2 que muestra la cantidad de publicaciones por mes.
#' @examples
#' \dontrun{
#' datos <- extraer_noticias_fecha("cambio climatico", "2024-01-01","2025-01-01", subir_a_bd = FALSE)
#' grafico_notas_por_mes(datos, titulo = "Cambio Climatico",
#' fecha_inicio = "2024-01-01", fecha_fin = "2024-06-06")
#' }
#'
#' @export

grafico_notas_por_mes <- function(datos, titulo, fecha_inicio = NULL, fecha_fin = NULL) {
  # Validar que 'datos' sea un data frame
  warning("La funcion 'grafico_notas_por_mes()' esta desactualizada. Se recomienda usar 'grafico_notas_fecha()'.")
  if (!is.data.frame(datos)) {
    stop("'datos' debe ser un data frame.")
  }
  # Validar que 'fecha' exista en los datos
  if (!"fecha" %in% colnames(datos)) {
    stop("'datos' debe contener la columna 'fecha'.")
  }

  # Validar que 'titulo' sea texto
  if (!is.character(titulo)) {
    stop("'titulo' debe ser texto (string).")
  }

  # Convertir 'fecha' a formato fecha
  datos <- datos %>%
    dplyr::mutate(fecha = as.Date(fecha, format = "%Y-%m-%d"))

  # Filtrar por rango de fechas si se especifican
  if (!is.null(fecha_inicio) || !is.null(fecha_fin)) {
    if (!is.null(fecha_inicio)) {
      fecha_inicio <- as.Date(fecha_inicio, format = "%Y-%m-%d")
    } else {
      fecha_inicio <- min(datos$fecha, na.rm = TRUE)
    }
    if (!is.null(fecha_fin)) {
      fecha_fin <- as.Date(fecha_fin, format = "%Y-%m-%d")
    } else {
      fecha_fin <- max(datos$fecha, na.rm = TRUE)
    }

    # Validar que las fechas sean validas
    if (fecha_inicio > fecha_fin) {
      stop("'fecha_inicio' debe ser anterior o igual a 'fecha_fin'.")
    }

    # Filtrar datos por rango
    datos_filtrados <- datos %>%
      dplyr::filter(fecha >= fecha_inicio & fecha <= fecha_fin)
  } else {
    # Usar todo el rango de fechas disponible
    datos_filtrados <- datos
  }

  # Verificar si hay datos despues del filtrado
  if (nrow(datos_filtrados) == 0) {
    stop("No hay datos en el rango de fechas seleccionado.")
  }

  # Agrupar datos por mes y contar las publicaciones
  publicaciones_por_mes <- datos_filtrados %>%
    dplyr::mutate(fecha = lubridate::floor_date(fecha, "month")) %>%
    dplyr::group_by(fecha) %>%
    dplyr::summarise(cantidad = dplyr::n(), .groups = "drop") %>%
    dplyr::arrange(fecha)

  # Generar el grafico
  grafico <- ggplot2::ggplot(publicaciones_por_mes, ggplot2::aes(x = fecha, y = cantidad)) +
    ggplot2::geom_line(color = "blue", linewidth = 1) +
    ggplot2::geom_point(color = "red", size = 1) +
    ggplot2::geom_smooth(method = "loess", color = "green", se = FALSE, linewidth = 1) +
    ggplot2::labs(
      title = titulo,
      x = "Fecha",
      y = "Cantidad de Notas"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))

  # Devolver el grafico
  return(grafico)
}

test_that("La función lanza un error si 'datos' no es un data frame", {
  expect_error(
    grafico_notas_por_mes("no es un data frame", "Título"),
    "'datos' debe ser un data frame."
  )
})

test_that("La función lanza un error si 'fecha' no está en los datos", {
  datos_sin_fecha <- data.frame(otra_columna = c(1, 2, 3))
  expect_error(
    grafico_notas_por_mes(datos_sin_fecha, "Título"),
    "'datos' debe contener la columna 'fecha'."
  )
})

test_that("La función lanza un error si 'titulo' no es texto", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01"))
  expect_error(
    grafico_notas_por_mes(datos, 123),
    regexp = "'titulo' debe ser texto \\(string\\)."
  )
})

test_that("La función lanza un error si 'fecha_inicio' es posterior a 'fecha_fin'", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01"))
  expect_error(
    grafico_notas_por_mes(datos, "Título", fecha_inicio = "2024-03-01", fecha_fin = "2024-01-01"),
    "'fecha_inicio' debe ser anterior o igual a 'fecha_fin'."
  )
})

test_that("La función lanza un error si no hay datos en el rango de fechas", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01"))
  expect_error(
    grafico_notas_por_mes(datos, "Título", fecha_inicio = "2025-01-01", fecha_fin = "2025-12-31"),
    "No hay datos en el rango de fechas seleccionado."
  )
})

test_that("La función genera un gráfico correctamente con datos válidos", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01", "2024-02-15", "2024-03-01"))
  grafico <- grafico_notas_por_mes(datos, "Título")
  expect_s3_class(grafico, "ggplot")
})

test_that("La función filtra correctamente por rango de fechas", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01", "2024-02-15", "2024-03-01"))
  grafico <- grafico_notas_por_mes(datos, "Título", fecha_inicio = "2024-02-01", fecha_fin = "2024-02-28")
  expect_s3_class(grafico, "ggplot")
})

test_that("Las fechas en el gráfico están dentro del rango especificado", {
  datos <- data.frame(fecha = c("2024-01-01", "2024-02-01", "2024-02-15", "2024-03-01", "2024-04-01"))

  # Generar gráfico con un rango de fechas específico
  grafico <- grafico_notas_por_mes(datos, "Título", fecha_inicio = "2024-02-01", fecha_fin = "2024-03-31")

  # Extraer las fechas del objeto gráfico
  fechas_grafico <- grafico$data$fecha

  # Comprobar que todas las fechas están dentro del rango
  expect_true(all(fechas_grafico >= as.Date("2024-02-01") & fechas_grafico <= as.Date("2024-03-31")))
})

#' Extraccion de noticias de BioBio.cl por cantidad maxima de resultados
#'
#' Esta funcion permite realizar una extraccion automatizada de noticias de BioBio.cl
#' entregando como parametro una cantidad maxima de resultados.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param max_results Cantidad maxima de resultados (opcional).
#' @return Un dataframe con las noticias extraidas.
#' @examples
#' \dontrun{
#' noticias <- extraer_noticias_fecha_bbcl("inteligencia artificial", "2025-01-01",
#' "2025-02-24")
#' }
#' @export
extraer_noticias_max_res_bbcl <- function(search_query, max_results = NULL) {

# Inicializamos variables
encoded_query <- URLencode(search_query)
all_data <- data.frame(
  ID = character(),
  post_title = character(),
  post_content = character(),
  post_content_clean = character(),
  post_URL = character(),
  post_categories = character(), # este
  post_tags = character(),       # y este los tenemos que unificar posteriormente
  post_image.URL = character(),
  author.display_name = character(),
  raw_post_date = as.Date(character()),
  resumen_de_ia = character(),
  search_query = character(),
  medio = character(), # en esta version solo es posible bbcl
  stringsAsFactors = FALSE
)

# Obtenemos la respuesta inicial
respuesta_inicial <- init_req_bbcl(search_query)
fecha_mas_reciente <- lubridate::ymd_hms(respuesta_inicial$raw_post_date[1])
total_results <- as.integer(respuesta_inicial$total)
if(total_results > 0){
  message(paste0("Total de resultados disponibles en bbcl: ", total_results))
  message(paste0("Noticia mas reciente disponible en bbcl es de la fecha: ", fecha_mas_reciente))
}else{
  stop("No se encontraron noticias con la search query especificada.")
}

# Determinamos el numero de resultados a extraer
if (is.null(max_results) || max_results > total_results) {
  max_results <- total_results
}

# Iteramos para obtener todas las noticias necesarias
offset <- 0
while (nrow(all_data) < max_results) {
  url <- paste0(
    "https://www.biobiochile.cl/lista/api/buscador?offset=", offset,
    "&search=", encoded_query,
    "&intervalo=&orden=ultimas"
  )

  response <- httr::GET(url, httr::add_headers(.headers = c(
    `User-Agent` = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:131.0) Gecko/20100101 Firefox/131.0",
    `Accept` = "application/json, text/plain, */*",
    `Content-Type` = "application/json; charset=UTF-8"
  )))

  if (response$status_code != 200) {
    warning("Error al realizar la solicitud. Codigo de estado: ", response$status_code)
    break
  }

  data <- httr::content(response, "text", encoding = "UTF-8") %>%
    jsonlite::fromJSON(flatten = TRUE)

  if (is.null(data$notas) || length(data$notas) == 0) {
    warning("No se encontraron mas notas para extraer.")
    break
  }

  noticias <- as.data.frame(data$notas)
  # Verificar las columnas de noticias
  missing_columns <- setdiff(colnames(all_data), colnames(noticias))

  # Si faltan columnas, anadirlas con valores NA
  for (col in missing_columns) {
    noticias[[col]] <- NA
  }

  # Asegurarse de que las columnas esten en el orden correcto
  noticias <- noticias[, colnames(all_data), drop = FALSE]
  # Anadir las noticias a all_data
  all_data <- rbind(all_data, noticias)

  # Controlar el numero de resultados
  if (nrow(all_data) >= max_results) {
    all_data <- all_data[1:max_results, ]
    break
  }

  offset <- offset + 20
}

all_data$search_query <- tolower(search_query)
all_data$raw_post_date <- as.Date(all_data$raw_post_date)
all_data$post_image.URL <- paste0("https://media.biobiochile.cl/wp-content/uploads/", as.character(all_data$post_image.URL))

# Creamos columna temas y eliminamos las que almacenaban data frames
# Creamos la nueva columna "temas" como una lista combinada
all_data$temas <- lapply(seq_len(nrow(all_data)), function(i) {
  # Extraer los slugs de post_categories
  slugs_categorias <- all_data$post_categories[[i]]$slug

  # Extraer los slugs de post_tags
  slugs_tags <- all_data$post_tags[[i]]$slug

  # Combinar ambos en un solo vector
  temas_combinados <- c(slugs_categorias, slugs_tags)

  # Devolver la lista combinada
  temas_combinados
})

# Eliminar columnas originales
all_data$post_categories <- NULL
all_data$post_tags <- NULL

# Definir contenido de la columa medio
all_data$medio <- "bbcl"

###############################

# Redefinir nombres de columnas

colnames(all_data) <- colnames(all_data) %>%
  dplyr::recode(
    post_title = "titulo",
    post_content = "contenido",
    post_URL = "url",
    `author.display_name` = "autor",
    raw_post_date = "fecha",
    resumen_de_ia = "resumen",
    post_content_clean = "contenido_limpio",
    `post_image.URL` = "url_imagen"
  )

###############################

return(all_data)
}

#' Parser de Fuentes
#'
#' Esta funcion toma un string que contiene nombres de fuentes separados por comas
#' y devuelve una lista con cada fuente como un elemento separado, sin espacios en blanco adicionales.
#'
#' @param cadena Un string que contiene nombres de fuentes separados por comas.
#' @return Una lista de strings, cada uno representando una fuente sin espacios en blanco adicionales.
#' @examples
#' parserFuentes("bbcl, emol, mediosregionales, ")
#' parserFuentes(" emol-todas, bbcl")
#' @export

parserFuentes <- function(cadena) {
  # Validacion: Verificar que el argumento no sea nulo
  if (is.null(cadena)) {
    stop("Error: El argumento 'cadena' no puede ser nulo.")
  }

  # Validacion: Verificar que el argumento sea un string
  if (!is.character(cadena) || length(cadena) != 1) {
    stop("Error: El argumento 'cadena' debe ser un string.")
  }

  # Validacion: Manejar el caso de una cadena vacia
  if (nchar(cadena) == 0) {
    return(list())
  }

  # Separar la cadena por comas
  elementos <- unlist(strsplit(cadena, split = ","))

  # Eliminar espacios en blanco adicionales de cada elemento
  elementos_limpios <- trimws(elementos)

  # Filtrar elementos no vacios
  elementos_no_vacios <- elementos_limpios[elementos_limpios != ""]

  # Definir los valores permitidos
  valores_permitidos <- c("bbcl", "emol", "guioteca", "mediosregionales", "emol-todas", "ciper")

  # Filtrar elementos que no estan en los valores permitidos
  elementos_validos <- elementos_no_vacios[elementos_no_vacios %in% valores_permitidos]

  # Si 'emol-todas' esta presente, eliminar 'emol', 'guioteca' y 'mediosregionales'
  if ("emol-todas" %in% elementos_validos) {
    elementos_validos <- elementos_validos[!elementos_validos %in% c("emol", "guioteca", "mediosregionales")]
  }

  # Retornar como lista
  return(as.list(elementos_validos))
}

# R/utils-emol-helpers.R

#' Helper para extraer la URL de la imagen de datos de Emol
#'
#' Esta funcion interna procesa las columnas relevantes de una fila de datos
#' de Emol para extraer la URL de la imagen principal de la noticia.
#' Prioriza `_source.tablas.tablaMedios` para fuentes "emol" y "mediosregionales",
#' y recurre a `_source.imagen` para otras o si la primera falla.
#' Maneja diferentes estructuras de datos (dataframes, listas, valores atomicos).
#'
#' @param source_tablas_medios El contenido de la columna `_source.tablas.tablaMedios`
#'   de la fila actual. Puede ser un dataframe, lista, o NULL.
#' @param source_imagen El contenido de la columna `_source.imagen` de la fila actual.
#'   Puede ser un string, lista, o NULL.
#' @param fuente_actual Un string que indica la fuente de la noticia (e.g., "emol",
#'   "guioteca", "mediosregionales").
#'
#' @return Un string (character) con la URL de la imagen, o `NA_character_` si
#'   no se puede extraer una URL valida o si ocurre un error.
#' @keywords internal
helper_extraer_url_imagen <- function(source_tablas_medios, source_imagen, fuente_actual) {
  url_img <- NA_character_
  tryCatch({
    if (fuente_actual %in% c("emol", "mediosregionales")) {
      if (is.data.frame(source_tablas_medios) && nrow(source_tablas_medios) > 0 && "Url" %in% names(source_tablas_medios)) {
        url_img <- as.character(source_tablas_medios[1, "Url"])
      }
    }

    # Fallback o para otras fuentes como guioteca
    if (is.na(url_img) || !(fuente_actual %in% c("emol", "mediosregionales"))) {
      if (!is.null(source_imagen) && !is.list(source_imagen) && !is.data.frame(source_imagen) && !is.na(source_imagen)) {
        url_img <- as.character(source_imagen)
      } else if (is.list(source_imagen) && length(source_imagen) > 0) {
        # Si es una lista, tomar el primer elemento si no es otra lista/df
        primer_elem <- source_imagen[[1]]
        if(!is.null(primer_elem) && !is.na(primer_elem) && !is.list(primer_elem) && !is.data.frame(primer_elem)){
          url_img <- as.character(primer_elem)
        }
      }
    }

    # Limpieza final: asegurar que no sea una lista, dataframe, o vector multiple
    if (is.list(url_img) || is.data.frame(url_img) || length(url_img) == 0) {
      url_img <- NA_character_
    } else if (length(url_img) > 1) {
      url_img <- url_img[1] # Tomar solo el primero si es un vector
    }
    if (is.character(url_img) && nzchar(url_img) == FALSE) url_img <- NA_character_ # Convertir "" a NA

  }, error = function(e) {
    url_img <- NA_character_
  })
  return(url_img)
}


#' Helper para extraer el resumen (bajada) de datos de Emol
#'
#' Esta funcion interna procesa la columna `_source.bajada` de una fila
#' de datos de Emol para extraer el texto del resumen.
#' Maneja casos donde `_source.bajada` puede ser NULL, un dataframe,
#' una lista, o un valor atomico.
#'
#' @param bajada_val El contenido de la columna `_source.bajada` de la fila actual.
#'
#' @return Un string (character) con el texto del resumen, o `NA_character_`
#'   si no se puede extraer o si ocurre un error.
#' @keywords internal
helper_extraer_resumen <- function(bajada_val) {
  resultado_resumen <- NA_character_
  tryCatch({
    if (!is.null(bajada_val)) {
      if (is.data.frame(bajada_val)) {
        if (nrow(bajada_val) > 0 && ncol(bajada_val) > 0) {
          val_celda <- bajada_val[1,1]
          if (!is.null(val_celda) && !is.na(val_celda) && !is.list(val_celda) && !is.data.frame(val_celda)) {
            resultado_resumen <- as.character(val_celda)
          }
        }
      } else if (is.list(bajada_val)) {
        if (length(bajada_val) > 0) {
          primer_elemento_lista <- bajada_val[[1]]
          if (is.data.frame(primer_elemento_lista)) {
            if (nrow(primer_elemento_lista) > 0 && ncol(primer_elemento_lista) > 0) {
              val_celda_lista <- primer_elemento_lista[1,1]
              if(!is.null(val_celda_lista) && !is.na(val_celda_lista) && !is.list(val_celda_lista) && !is.data.frame(val_celda_lista)){
                resultado_resumen <- as.character(val_celda_lista)
              }
            }
          } else if (!is.list(primer_elemento_lista)) { # Evitar listas anidadas de forma recursiva simple
            if(!is.null(primer_elemento_lista) && !is.na(primer_elemento_lista)){
              resultado_resumen <- as.character(primer_elemento_lista)
            }
          }
        }
      } else if (!is.na(bajada_val)) { # Valor atomico
        resultado_resumen <- as.character(bajada_val)
      }
    }

    # Limpieza final
    if (is.list(resultado_resumen) || is.data.frame(resultado_resumen) || length(resultado_resumen) == 0) {
      resultado_resumen <- NA_character_
    } else if (length(resultado_resumen) > 1) {
      resultado_resumen <- resultado_resumen[1]
    }
    if (is.character(resultado_resumen) && nzchar(resultado_resumen) == FALSE) resultado_resumen <- NA_character_

  }, error = function(e) {
    resultado_resumen <- NA_character_
  })
  return(resultado_resumen)
}


#' Helper para extraer y procesar los temas de datos de Emol
#'
#' Esta funcion interna procesa las columnas `_source.seccion`,
#' `_source.subSeccion` (si aplica), y `_source.temas` de una fila de
#' datos de Emol para generar una lista de temas procesados.
#' Los temas se convierten a minusculas y los espacios se reemplazan por guiones.
#'
#' @param source_seccion El contenido de la columna `_source.seccion`.
#' @param source_subseccion El contenido de la columna `_source.subSeccion`.
#' @param source_temas El contenido de la columna `_source.temas`. Puede ser
#'   un dataframe, lista, vector, o NULL.
#' @param fuente_actual Un string que indica la fuente (e.g., "mediosregionales").
#'
#' @return Una lista que contiene un unico elemento: un vector de strings (character)
#'   con los temas procesados y unicos. Si no hay temas, devuelve una lista
#'   conteniendo un vector de caracteres vacio (`character(0)`).
#' @keywords internal
helper_extraer_temas <- function(source_seccion, source_subseccion, source_temas, fuente_actual) {
  temas_lista_final <- list(character(0)) # Valor por defecto
  tryCatch({
    t_vector <- c()

    # Funcion interna para anadir terminos validos al vector de temas
    anadir_termino <- function(vec, termino) {
      if (!is.null(termino) && !is.na(termino) && !is.list(termino) && !is.data.frame(termino) && nzchar(as.character(termino))) {
        vec <- c(vec, as.character(termino))
      }
      return(vec)
    }

    if (fuente_actual == "mediosregionales") {
      t_vector <- anadir_termino(t_vector, source_seccion)
      t_vector <- anadir_termino(t_vector, source_subseccion)
    } else {
      t_vector <- anadir_termino(t_vector, source_seccion)
    }

    # Manejo de source_temas
    if (!is.null(source_temas)) {
      if (is.data.frame(source_temas) && nrow(source_temas) > 0 && "nombre" %in% names(source_temas)) {
        nombres_temas <- source_temas$nombre
        for(nombre_tema in nombres_temas) {
          t_vector <- anadir_termino(t_vector, nombre_tema)
        }
      } else if (is.list(source_temas) && length(source_temas) > 0) {
        elementos_temas_procesados <- unlist(lapply(source_temas, function(x_tema) {
          if(!is.list(x_tema) && !is.data.frame(x_tema) && !is.null(x_tema) && !is.na(x_tema) && nzchar(as.character(x_tema))) {
            as.character(x_tema)
          } else {
            NULL # Omitir elementos no validos o complejos
          }
        }))
        t_vector <- c(t_vector, elementos_temas_procesados)
      } else if (!is.list(source_temas) && !is.data.frame(source_temas)) { # Si es un vector atomico
        t_vector <- anadir_termino(t_vector, source_temas) # Esto podria anadir multiples si source_temas es un vector
        # Si source_temas es un vector, y anadir_termino solo toma el primer elemento implicitamente,
        # se podria necesitar iterar: for(st in source_temas) { t_vector <- anadir_termino(t_vector, st) }
        # Pero anadir_termino ya hace c(vec, as.character(termino)), lo cual maneja vectores.
      }
    }

    if (length(t_vector) > 0) {
      t_vector_limpio <- tolower(t_vector[!is.na(t_vector) & t_vector != "" & sapply(t_vector, nzchar)])
      t_vector_limpio <- gsub("\\s+", "-", t_vector_limpio)
      temas_lista_final <- list(unique(t_vector_limpio[nzchar(t_vector_limpio)]))
    }
    # Si temas_lista_final sigue siendo list(character(0)) despues de esto, esta bien.

  }, error = function(e) {
    # En caso de error, devuelve una lista con un vector de caracteres vacio
    temas_lista_final <- list(character(0))
  })
  return(temas_lista_final)
}

#' Inicializa una solicitud a emol.com y retorna maximo 10 noticias
#'
#' Esta funcion auxiliar llama a emol.com utilizando una frase de busqueda.
#' Entrega como maximo 10 resultados. Se debe llamar desde otras funciones solo
#' con una fuente a la vez, es decir, sin llamar a emol-todas.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param page La pagina de busqueda para iterar, es un int
#' @param fuentes Es un string que deberia tener solo fuentes de emol posibles separadas por comas.
#' @return Un dataframe con el caso de la busqueda, incluyendo solo columnas especificas.
#' @examples
#' \dontrun{
#' primer_caso <- iteracion_emol("Boric", fuentes="emol-todas")
#' }
#' @export

iteracion_emol <- function(search_query, page=0, fuentes="emol-todas") {
  # Validamos el parametro
  if (missing(search_query) || !is.character(search_query)) {
    stop("Debe proporcionar una frase de busqueda como texto.")
  }
  # Hay que validar el formato del string fuente

  # Parametros de configuracion y adicionales
  batch_size <- 10
  data_initial <- list()
  patronBusqueda <- ""

  if(fuentes=="emol-todas"){
    patronBusqueda <- "emol,mediosregionales,guioteca"
  } else {
    fuentesParsed <- parserFuentes(fuentes)
    if(length(fuentesParsed) > 0){
      patronBusqueda <- paste0(fuentesParsed, collapse = ",")
    } else {
      message("Error al parsear las fuentes")
    }
  }


  # URL iteracion
  url_iteracion <- paste0(
    "https://newsapi.ecn.cl/NewsApi/emol/buscador/",
    patronBusqueda,
    "?q=",URLencode(search_query),
    "&size=10&from=", (page*batch_size)
  )

  # Solicitud iteracion
  response_iteracion <- httr::GET(url_iteracion)

  if (response_iteracion$status_code == 200) {
    data_iteracion <- httr::content(response_iteracion, "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON(flatten = TRUE)

    if (!is.null(data_iteracion$hits$hits) && length(data_iteracion$hits$hits) > 0) {
      datos_iteracion <- as.data.frame(data_iteracion$hits$hits)

      columnas_deseadas <- c(
        "_id",
        "_source.titulo",
        "_source.texto",
        "_source.permalink",
        "_source.tablas.tablaMedios",
        "_source.imagen",
        "_source.autor",
        "_source.bajada",
        "_source.seccion",
        "_source.subSeccion",
        "_source.fechaPublicacion",
        "_source.fechaModificacion"
      )

      # Filtrar solo las columnas que existen en el dataframe
      columnas_disponibles <- columnas_deseadas[columnas_deseadas %in% names(datos_iteracion)]

      # Si no hay columnas disponibles, mostrar advertencia
      if(length(columnas_disponibles) == 0) {
        warning("Ninguna de las columnas especificadas esta disponible en los datos.")
        return(datos_iteracion)
      }

      # Retornamos el dataframe con solo las columnas seleccionadas
      return(datos_iteracion[, columnas_disponibles])
    } else {
      stop("No se encontraron notas en la respuesta de la iteracion.")
    }
  } else {
    stop("Error al realizar la solicitud inicial. Codigo de estado: ", response_iteracion$status_code)
  }
}

#' @importFrom utils URLencode
#' @importFrom rlang .data
#' @export
utils::globalVariables(c(
  "contenido", "fecha", "cantidad", "word",
  "contenido_limpio", "n", "ID", "_id", "_source.autor",
  "_source.bajada", "_source.imagen", "_source.permalink",
  "_source.seccion", "_source.subSeccion", "_source.tablas.tablaMedios",
  "_source.temas", "_source.texto", "_source.titulo", "autor", "medio",
  "resumen", "temas", "titulo", "url_imagen", "fecha_grupo", "fecha_texto"
))

#' Funcion para limpiar notas de contenido HTML
#'
#' Esta funcion permite limpiar por completo las notas eliminando codigos y secciones irrelevantes.
#' Verifica que el input sea un data frame con una columna llamada `contenido`.
#' @param datos Data frame donde estan almacenadas las notas
#' @param sinonimos Un vector de character
#' @return Un dataframe con el contenido limpio en la columna contenido_limpio
#' @examples
#'
#' \donttest{
#' datos <- extraer_noticias_max_res("inteligencia artificial", max_results= 150, subir_a_bd = FALSE)
#' datos_proc <- limpieza_notas(datos, sinonimos = c("IA", "AI"))
#' }
#' @export

limpieza_notas <- function(datos, sinonimos = c()) {
  # Validacion inicial
  if (!is.data.frame(datos)) {
    stop("El argumento 'datos' debe ser un data frame.")
  }
  if (!"contenido" %in% colnames(datos)) {
    stop("El data frame debe contener una columna llamada 'contenido'.")
  }
  if (!is.null(sinonimos) && !is.character(sinonimos)) {
    stop("'sinonimos' debe ser un vector de palabras.")
  }

  # Sobrescribir/crear contenido_limpio a partir de contenido
  datos$contenido_limpio <- datos$contenido

  # Iteramos sobre cada fila del data frame
  for (i in seq_len(nrow(datos))) {
    current_content <- datos$contenido_limpio[[i]]

    is_html_like <- FALSE

    if (!is.na(current_content) && is.character(current_content) && nzchar(current_content)) {
      is_html_like <- stringr::str_detect(current_content, "<\\s*(div|p|span|a|body|html|br|img|strong|em|h[1-6])[^>]*>")
    }

    if (is_html_like) {
      parsed_html <- tryCatch({
        rvest::read_html(current_content)
      }, error = function(e) {
        return(NULL)
      })

      if (!is.null(parsed_html)) {
        parsed_html %>%
          rvest::html_nodes("div.lee-tambien-bbcl") %>%
          xml2::xml_remove()
        parsed_html %>%
          rvest::html_nodes("blockquote.instagram-media") %>%
          xml2::xml_remove()
        parsed_html %>%
          rvest::html_nodes("blockquote.twitter-tweet") %>%
          xml2::xml_remove()

        processed_text <- rvest::html_text2(parsed_html)
      } else {
        processed_text <- current_content
      }
    } else {
      processed_text <- current_content
    }

    if (!is.na(processed_text)) {
      processed_text <- stringr::str_replace_all(
        processed_text,
        stringr::regex("Lee tambi\u00e9n.*?(<\\/div>)?", dotall = TRUE, ignore_case = TRUE),
        ""
      )
      datos$contenido_limpio[[i]] <- stringr::str_squish(processed_text)
    } else {
      datos$contenido_limpio[[i]] <- NA
    }
  }

  main_query <- ""
  if ("search_query" %in% colnames(datos) && nrow(datos) > 0 && !is.na(datos$search_query[[1]])) {
    main_query <- datos$search_query[[1]]
  }

  escape_regex <- function(string) {
    if(is.na(string)) return(NA_character_)
    stringr::str_replace_all(string, "([.\\\\+*?\\[\\^\\]$(){}=!<>|:])", "\\\\\\1")
  }

  terms_to_match <- c()
  if (main_query != "" && !is.na(main_query)) {
    terms_to_match <- c(terms_to_match, escape_regex(main_query))
  }
  if (length(sinonimos) > 0) {
    valid_sinonimos <- sinonimos[!is.na(sinonimos)]
    if(length(valid_sinonimos) > 0) {
      terms_to_match <- c(terms_to_match, sapply(valid_sinonimos, escape_regex))
    }
  }

  if (length(terms_to_match) > 0) {
    pattern <- paste0("(?i)\\b(", paste(terms_to_match, collapse = "|"), ")\\b")
    matches_pattern <- stringr::str_detect(datos$contenido_limpio, pattern)
    indices_no_match <- which(matches_pattern == FALSE)


    if(length(indices_no_match) > 0 && length(indices_no_match) < nrow(datos)) {
      datos <- datos[-indices_no_match, , drop = FALSE]
    } else if (length(indices_no_match) == nrow(datos) && nrow(datos) > 0) {
      original_colnames <- colnames(datos)
      datos <- datos[0, , drop = FALSE]
      colnames(datos) <- original_colnames
    }
  }

  if (nrow(datos) == 0) {
    return(datos)
  }

  return(datos)
}

#' Inicializa una solicitud a BioBio.cl y retorna el primer caso de busqueda
#'
#' Esta funcion permite realizar una consulta inicial a BioBio.cl utilizando una frase de busqueda.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @return Un dataframe con el primer caso de la busqueda.
#' @export
#' @examples
#' \dontrun{
#' primer_caso <- init_req_bbcl("inteligencia artificial")
#' }

init_req_bbcl <- function(search_query) {
  # Validamos el parametro
  if (missing(search_query) || !is.character(search_query)) {
    stop("Debe proporcionar una frase de busqueda como texto.")
  }

  # Encabezados para la solicitud
  headers <- c(
    `User-Agent` = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:131.0) Gecko/20100101 Firefox/131.0",
    `Accept` = "application/json, text/plain, */*",
    `Referer` = paste0("https://www.biobiochile.cl/buscador.shtml?s=", URLencode(search_query)),
    `Content-Type` = "application/json; charset=UTF-8"
  )

  # URL inicial
  url_initial <- paste0(
    "https://www.biobiochile.cl/lista/api/buscador?offset=0",
    "&search=", URLencode(search_query),
    "&intervalo=&orden=ultimas"
  )

  # Solicitud inicial
  response_initial <- httr::GET(url_initial, httr::add_headers(.headers = headers))
  if (response_initial$status_code == 200) {
    data_initial <- httr::content(response_initial, "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON(flatten = TRUE)

    if (!is.null(data_initial$notas) && length(data_initial$notas) > 0) {
      # Extraemos el primer caso
      first_case <- as.data.frame(data_initial$notas[1, ])

      # Agregamos la columna "total"
      first_case$total <- data_initial$total

      # Retornamos el primer caso
      return(first_case)
    } else {
      stop("No se encontraron notas en la respuesta inicial.")
    }
  } else {
    stop("Error al realizar la solicitud inicial. Codigo de estado: ", response_initial$status_code)
  }
}

testthat::test_that("Se entrega un data frame de una sola fila, mostrando la noticia más reciente sobre la búsqueda", {

  primer_caso <- init_req_bbcl("inteligencia artificial")

  #Verificamos que es data frame
  expect_s3_class(primer_caso, "data.frame")

  #Verificamos la cantidad de resultados
  expect_equal(nrow(primer_caso), 1)

  #Verificamos la existencia de las columnas deseadas
  expect_true("ID" %in% colnames(primer_caso))
  expect_true("post_title" %in% colnames(primer_caso))
  expect_true("post_content" %in% colnames(primer_caso))
  expect_true("post_excerpt" %in% colnames(primer_caso))
  expect_true("post_URL" %in% colnames(primer_caso))
  expect_true("post_categories" %in% colnames(primer_caso))
  expect_true("post_tags" %in% colnames(primer_caso))
  expect_true("year" %in% colnames(primer_caso))
  expect_true("month" %in% colnames(primer_caso))
  expect_true("day" %in% colnames(primer_caso))
  expect_true("post_category_primary.name" %in% colnames(primer_caso))
  expect_true("post_category_secondary.name" %in% colnames(primer_caso))
  expect_true("post_image.URL" %in% colnames(primer_caso))
  expect_true("post_image.alt" %in% colnames(primer_caso))
  expect_true("post_image.caption" %in% colnames(primer_caso))
  expect_true("author.display_name" %in% colnames(primer_caso))
  expect_true("raw_post_date" %in% colnames(primer_caso))
  expect_true("resumen_de_ia" %in% colnames(primer_caso))









})

#' Extraccion de noticias de BioBio.cl por rango de fechas
#'
#' Esta funcion permite realizar una extraccion automatizada de noticias de BioBio.cl utilizando un rango de fechas.
#'
#' @param search_query Una frase de busqueda (obligatoria).
#' @param fecha_inicio Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @param fecha_fin Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).
#' @return Un dataframe con las noticias extraidas.
#' @examples
#' \dontrun{
#' noticias <- extraer_noticias_fecha_bbcl("inteligencia artificial", "2025-01-01",
#' "2025-02-24")
#' }
#' @export

extraer_noticias_fecha_bbcl <- function(search_query, fecha_inicio, fecha_fin) {

  # Inicializamos variables
  offset <- 0
  total_results <- 0
  all_data <- data.frame(
    ID = character(),
    post_title = character(),
    post_content = character(),
    post_content_clean = character(),
    post_URL = character(),
    post_categories = character(), # este
    post_tags = character(),       # y este los tenemos que unificar posteriormente
    post_image.URL = character(),
    author.display_name = character(),
    raw_post_date = as.Date(character()),
    resumen_de_ia = character(),
    search_query = character(),
    medio = character(),
    stringsAsFactors = FALSE
  )

  # Encabezados para la solicitud
  headers <- c(
    `User-Agent` = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:131.0) Gecko/20100101 Firefox/131.0",
    `Accept` = "application/json, text/plain, */*",
    `Referer` = paste0("https://www.biobiochile.cl/buscador.shtml?s=", URLencode(search_query)),
    `Content-Type` = "application/json; charset=UTF-8"
  )

  # Obtenemos la respuesta inicial
  respuesta_inicial <- init_req_bbcl(search_query)
  fecha_mas_reciente <- lubridate::ymd_hms(respuesta_inicial$raw_post_date[1])
  total_results <- as.integer(respuesta_inicial$total)
  if(total_results > 0){
    message(paste0("Total de resultados posibles: ", total_results))
    message(paste0("Noticia mas reciente disponible es de la fecha: ", fecha_mas_reciente))
  }else{
    stop("No se encontraron noticias con la search query especificada.")
  }

  ## Bucle para iterar sobre las paginas de resultados
  repeat {
    # URL de solicitud
    url <- paste0(
      "https://www.biobiochile.cl/lista/api/buscador?offset=", offset,
      "&search=", utils::URLencode(search_query),
      "&intervalo=&orden=ultimas"
    )

    # Solicitud a la API
    response <- httr::GET(url, httr::add_headers(.headers = headers))
    if (response$status_code != 200) {
      stop("Error en la solicitud: codigo de estado ", response$status_code)
    }

    # Parseo de la respuesta
    data <- httr::content(response, "text", encoding = "UTF-8") %>%
      jsonlite::fromJSON(flatten = TRUE)

    # Salimos del bucle si no hay mas datos
    if (is.null(data$notas) || length(data$notas) == 0) break

    # Convertimos las fechas de las noticias
    data$notas$raw_post_date <- as.Date(data$notas$raw_post_date)

    # Filtramos las noticias dentro del rango de fechas
    noticias_filtradas <- data$notas[data$notas$raw_post_date >= as.Date(fecha_inicio) &
                                       data$notas$raw_post_date <= as.Date(fecha_fin), ]

    # Verificamos si hay noticias filtradas
    if (nrow(noticias_filtradas) > 0) {
      # Aseguramos que solo las columnas necesarias esten presentes
      # Seleccionamos solo las columnas que existen en all_data
      noticias_filtradas <- noticias_filtradas[, intersect(names(noticias_filtradas), names(all_data))]

      # Si hay columnas faltantes en noticias_filtradas, las agregamos como NA
      columnas_faltantes <- setdiff(names(all_data), names(noticias_filtradas))
      if (length(columnas_faltantes) > 0) {
        for (col in columnas_faltantes) {
          noticias_filtradas[[col]] <- NA  # Agregamos columnas faltantes como NA
        }
      }

      # Reordenamos las columnas de noticias_filtradas para que coincidan con el orden de all_data
      noticias_filtradas <- noticias_filtradas[names(all_data)]

      # Agregamos noticias filtradas al dataframe all_data
      all_data <- rbind(all_data, noticias_filtradas)

    } else {
      fecha_reciente <- max(data$notas$raw_post_date)
      if (fecha_reciente < as.Date(fecha_inicio)) {
        message("No hay mas noticias dentro del rango de fechas. Terminando la busqueda.")
        break  # Salimos del bucle si la fecha mas reciente es anterior a fecha_inicio
      }
    }

    # Incrementamos offset
    offset <- offset + 20

    # Salimos si ya hemos procesado todos los resultados disponibles
    if (offset >= total_results) break
  }

  all_data$search_query <- tolower(search_query)

  all_data$post_image.URL <- paste0("https://media.biobiochile.cl/wp-content/uploads/", as.character(all_data$post_image.URL))

  message(paste0("Total de noticias encontradas en el rango de fechas: ", nrow(all_data)))

  # Creamos columna temas y eliminamos las que almacenaban data frames
  # Creamos la nueva columna "temas" como una lista combinada
  all_data$temas <- lapply(seq_len(nrow(all_data)), function(i) {
    # Extraer los slugs de post_categories
    slugs_categorias <- all_data$post_categories[[i]]$slug

    # Extraer los slugs de post_tags
    slugs_tags <- all_data$post_tags[[i]]$slug

    # Combinar ambos en un solo vector
    temas_combinados <- c(slugs_categorias, slugs_tags)

    # Devolver la lista combinada
    temas_combinados
  })

  # Eliminar columnas originales
  all_data$post_categories <- NULL
  all_data$post_tags <- NULL

  # Definir contenido de la columa medio
  all_data$medio <- "bbcl"

  ###############################

  # Redefinir nombres de columnas

  colnames(all_data) <- colnames(all_data) %>%
    dplyr::recode(
      post_title = "titulo",
      post_content = "contenido",
      post_URL = "url",
      `author.display_name` = "autor",
      raw_post_date = "fecha",
      resumen_de_ia = "resumen",
      post_content_clean = "contenido_limpio",
      `post_image.URL` = "url_imagen"
    )

  ###############################


  return(all_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parserFuentes.R
\name{parserFuentes}
\alias{parserFuentes}
\title{Parser de Fuentes}
\usage{
parserFuentes(cadena)
}
\arguments{
\item{cadena}{Un string que contiene nombres de fuentes separados por comas.}
}
\value{
Una lista de strings, cada uno representando una fuente sin espacios en blanco adicionales.
}
\description{
Esta funcion toma un string que contiene nombres de fuentes separados por comas
y devuelve una lista con cada fuente como un elemento separado, sin espacios en blanco adicionales.
}
\examples{
parserFuentes("bbcl, emol, mediosregionales, ")
parserFuentes(" emol-todas, bbcl")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_max_res_ciper.R
\name{extraer_noticias_max_res_ciper}
\alias{extraer_noticias_max_res_ciper}
\title{Extrae noticias de Ciper Chile}
\usage{
extraer_noticias_max_res_ciper(search_query = NULL, max_results = NULL)
}
\arguments{
\item{search_query}{El termino de busqueda (opcional). Si es NULL, extrae todos los articulos.}

\item{max_results}{El numero maximo de noticias a extraer.}
}
\value{
Un dataframe con las noticias extraidas, estandarizado al formato de datamedios.
}
\description{
Esta funcion se conecta a la API de Ciper para descargar noticias.
}
\examples{
\donttest{
# Extraer los ultimos 5 articulos con una busqueda
noticias_ciper <- extraer_noticias_max_res_ciper("corrupcion", max_results = 5)

# Extraer los ultimos 10 articulos sin busqueda
ultimos_ciper <- extraer_noticias_max_res_ciper(max_results = 10)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/word_cloud.R
\name{word_cloud}
\alias{word_cloud}
\title{Funcion de nube de palabras}
\usage{
word_cloud(datos, max_words, stop_words = NULL)
}
\arguments{
\item{datos}{data frame que incluye la columna contenido_limpio.}

\item{max_words}{Cantidad maxima de palabras que apareceran en la nube.}

\item{stop_words}{Definir las palabras que seran ignoradas en la visualizacion. Debe ser un vector de carácteres.}
}
\value{
Una nube de palabras con las palabras mas frecuentes.
}
\description{
Esta funcion permite realizar una nube de palabras con las palabras más frecuentes del corpus de noticias.
}
\examples{
\dontrun{
datos <- extraer_noticias_fecha("Boric",
"2025-03-01",
"2025-04-01",
fuentes="bbcl",
subir_a_bd = FALSE)
datos_proc <- limpieza_notas(datos)
word_cloud(datos_proc, max_words = 50, stop_words = c("es", "la"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-emol-helpers.R
\name{helper_extraer_url_imagen}
\alias{helper_extraer_url_imagen}
\title{Helper para extraer la URL de la imagen de datos de Emol}
\usage{
helper_extraer_url_imagen(source_tablas_medios, source_imagen, fuente_actual)
}
\arguments{
\item{source_tablas_medios}{El contenido de la columna `_source.tablas.tablaMedios`
de la fila actual. Puede ser un dataframe, lista, o NULL.}

\item{source_imagen}{El contenido de la columna `_source.imagen` de la fila actual.
Puede ser un string, lista, o NULL.}

\item{fuente_actual}{Un string que indica la fuente de la noticia (e.g., "emol",
"guioteca", "mediosregionales").}
}
\value{
Un string (character) con la URL de la imagen, o `NA_character_` si
  no se puede extraer una URL valida o si ocurre un error.
}
\description{
Esta funcion interna procesa las columnas relevantes de una fila de datos
de Emol para extraer la URL de la imagen principal de la noticia.
Prioriza `_source.tablas.tablaMedios` para fuentes "emol" y "mediosregionales",
y recurre a `_source.imagen` para otras o si la primera falla.
Maneja diferentes estructuras de datos (dataframes, listas, valores atomicos).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init_req_bbcl.R
\name{init_req_bbcl}
\alias{init_req_bbcl}
\title{Inicializa una solicitud a BioBio.cl y retorna el primer caso de busqueda}
\usage{
init_req_bbcl(search_query)
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}
}
\value{
Un dataframe con el primer caso de la busqueda.
}
\description{
Esta funcion permite realizar una consulta inicial a BioBio.cl utilizando una frase de busqueda.
}
\examples{
\dontrun{
primer_caso <- init_req_bbcl("inteligencia artificial")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_fecha_bbcl.R
\name{extraer_noticias_fecha_bbcl}
\alias{extraer_noticias_fecha_bbcl}
\title{Extraccion de noticias de BioBio.cl por rango de fechas}
\usage{
extraer_noticias_fecha_bbcl(search_query, fecha_inicio, fecha_fin)
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}

\item{fecha_inicio}{Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}

\item{fecha_fin}{Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}
}
\value{
Un dataframe con las noticias extraidas.
}
\description{
Esta funcion permite realizar una extraccion automatizada de noticias de BioBio.cl utilizando un rango de fechas.
}
\examples{
\dontrun{
noticias <- extraer_noticias_fecha_bbcl("inteligencia artificial", "2025-01-01",
"2025-02-24")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraccion_parrafos.R
\name{extraccion_parrafos}
\alias{extraccion_parrafos}
\title{Extraer parrafos de una columna de texto}
\usage{
extraccion_parrafos(datos, sinonimos = c())
}
\arguments{
\item{datos}{Data frame que contiene los datos de entrada con la columna "contenido".}

\item{sinonimos}{Vector de sinonimos que se incluiran en la busqueda.}
}
\value{
Data frame con una columna adicional 'parrafos_filtrados' que contiene los parrafos extraidos como listas.
}
\description{
Esta funcion procesa una columna de texto en un dataframe y extrae los parrafos que coinciden con los sinonimos proporcionados.
}
\examples{
\donttest{
datos <- extraer_noticias_max_res("inteligencia artificial", max_results = 140, subir_a_bd = FALSE)
datos <- extraccion_parrafos(datos, sinonimos = c("IA", "AI"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabla_frecuencia_palabras.R
\name{tabla_frecuencia_palabras}
\alias{tabla_frecuencia_palabras}
\title{Generar una tabla estilizada con las palabras mas frecuentes}
\usage{
tabla_frecuencia_palabras(datos, max_words, stop_words = NULL)
}
\arguments{
\item{datos}{Data frame que contiene la columna 'contenido_limpio'.}

\item{max_words}{Numero maximo de palabras que se mostraran en la tabla.}

\item{stop_words}{Vector opcional de palabras que se deben excluir del conteo.}
}
\value{
Una tabla con las palabras mas frecuentes.
}
\description{
Esta funcion procesa la columna 'contenido_limpio' de un dataframe, tokeniza el texto,
cuenta la frecuencia de cada palabra y genera una tabla con las palabras mas frecuentes.
}
\examples{
datos <- data.frame(
  contenido_limpio = c(
    "La ministra de Defensa Maya Fernandez enfrenta cuestionamientos
    el presidente Gabriel Boric solicita transparencia en los procesos.
    Renovacion Nacional pide la renuncia de Maya Fernandez debido a la polemica.
    La transparencia es fundamental en la politica y la gestion publica"
  ),
  stringsAsFactors = FALSE
)

# Probar la funcion con el dataframe de ejemplo
tabla_frecuencia_palabras(datos, max_words = 5, stop_words = c())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-emol-helpers.R
\name{helper_extraer_resumen}
\alias{helper_extraer_resumen}
\title{Helper para extraer el resumen (bajada) de datos de Emol}
\usage{
helper_extraer_resumen(bajada_val)
}
\arguments{
\item{bajada_val}{El contenido de la columna `_source.bajada` de la fila actual.}
}
\value{
Un string (character) con el texto del resumen, o `NA_character_`
  si no se puede extraer o si ocurre un error.
}
\description{
Esta funcion interna procesa la columna `_source.bajada` de una fila
de datos de Emol para extraer el texto del resumen.
Maneja casos donde `_source.bajada` puede ser NULL, un dataframe,
una lista, o un valor atomico.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grafico_notas_fecha.R
\name{grafico_notas_fecha}
\alias{grafico_notas_fecha}
\title{Grafico de notas por periodo (interactivo)}
\usage{
grafico_notas_fecha(
  datos,
  titulo,
  fecha_inicio = NULL,
  fecha_fin = NULL,
  agrupar_por = "day",
  tema = "light"
)
}
\arguments{
\item{datos}{Data frame con los datos procesados, que debe incluir la columna `fecha` en formato YYYY-MM-DD.}

\item{titulo}{Texto que aparecera en el titulo del grafico.}

\item{fecha_inicio}{Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).}

\item{fecha_fin}{Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).}

\item{agrupar_por}{Cadena de texto que especifica el periodo de agrupacion.
Valores validos son `"day"` (por defecto) o `"month"`.}

\item{tema}{Tema del grafico. Valores validos son `"light"` (por defecto) o `"dark"`.}
}
\value{
Un grafico plotly interactivo que muestra la cantidad de publicaciones por el periodo seleccionado.
}
\description{
Esta funcion genera un grafico interactivo que muestra la cantidad de publicaciones
agrupadas por dia o por mes, con opcion de tema dark o light.
}
\examples{
\dontrun{
# Ejemplo con tema dark, agrupando por mes
datos <- extraer_noticias_fecha("cambio climatico", "2024-01-01","2025-01-01", subir_a_bd = FALSE)
grafico_notas_fecha(datos, titulo = "Cambio Climatico (por mes)",
                    agrupar_por = "month", tema = "dark")

# Ejemplo con tema light, agrupando por dia
grafico_notas_fecha(datos, titulo = "Cambio Climatico (por dia)",
                    fecha_inicio = "2024-01-01", fecha_fin = "2024-03-31",
                    tema = "light")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grafico_notas_por_mes.R
\name{grafico_notas_por_mes}
\alias{grafico_notas_por_mes}
\title{Grafico de notas por mes}
\usage{
grafico_notas_por_mes(datos, titulo, fecha_inicio = NULL, fecha_fin = NULL)
}
\arguments{
\item{datos}{Data frame con los datos procesados, que debe incluir la columna `fecha` en formato YYYY-MM-DD.}

\item{titulo}{Texto que aparecera en el titulo del grafico.}

\item{fecha_inicio}{Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).}

\item{fecha_fin}{Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).}
}
\value{
Un grafico ggplot2 que muestra la cantidad de publicaciones por mes.
}
\description{
Esta funcion genera un grafico de linea que muestra la cantidad de publicaciones agrupadas por mes.
}
\examples{
\dontrun{
datos <- extraer_noticias_fecha("cambio climatico", "2024-01-01","2025-01-01", subir_a_bd = FALSE)
grafico_notas_por_mes(datos, titulo = "Cambio Climatico",
fecha_inicio = "2024-01-01", fecha_fin = "2024-06-06")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grafico_comparacion_medios.R
\name{grafico_comparacion_medios}
\alias{grafico_comparacion_medios}
\title{Grafico de comparacion de medios por periodo (Interactivo)}
\usage{
grafico_comparacion_medios(
  datos,
  titulo,
  fecha_inicio = NULL,
  fecha_fin = NULL,
  medios = NULL,
  agrupar_por = "day",
  tema = "light",
  tipo_grafico = "lineas"
)
}
\arguments{
\item{datos}{Data frame con los datos procesados, que debe incluir las columnas `fecha` y `medio`.}

\item{titulo}{Texto que aparecera en el titulo del grafico.}

\item{fecha_inicio}{Fecha de inicio para la construccion del grafico en formato YYYY-MM-DD (opcional).}

\item{fecha_fin}{Fecha de finalizacion para la construccion del grafico en formato YYYY-MM-DD (opcional).}

\item{medios}{Vector de strings con las medios a comparar. Si es NULL, usa todas las medios disponibles.}

\item{agrupar_por}{Cadena de texto que especifica el periodo de agrupacion.
Valores validos son `"day"` (por defecto) o `"month"`.}

\item{tema}{Tema del grafico. Valores validos son `"light"` (por defecto) o `"dark"`.}

\item{tipo_grafico}{Tipo de visualizacion. Valores validos son `"lineas"` (por defecto) o `"barras"`.}
}
\value{
Un grafico plotly interactivo que muestra la comparacion de publicaciones por medio y periodo.
}
\description{
Esta funcion genera un grafico interactivo que compara la cantidad de publicaciones
entre diferentes medios de medios, agrupadas por dia o por mes, con opcion de tema dark o light.
}
\examples{
\dontrun{
# Comparar todas las medios por mes
datos <- extraer_noticias_fecha("delincuencia", "2024-01-01", "2025-01-01", subir_a_bd = FALSE)
grafico_comparacion_medios(datos, titulo = "Cobertura de Delincuencia por Medio",
                            agrupar_por = "month", tema = "dark")

# Comparar medios especificas por dia
grafico_comparacion_medios(datos, titulo = "Comparacion BBCl vs emol",
                            medios = c("bbcl", "emol"),
                            fecha_inicio = "2024-06-01", fecha_fin = "2024-06-30",
                            agrupar_por = "day", tipo_grafico = "barras")
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_max_res.R
\name{extraer_noticias_max_res}
\alias{extraer_noticias_max_res}
\title{Extraccion de noticias de medios chilenos por cantidad maxima de resultados}
\usage{
extraer_noticias_max_res(
  search_query = NULL,
  max_results = NULL,
  subir_a_bd = TRUE,
  fuentes = "todas"
)
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}

\item{max_results}{Numero maximo de resultados a extraer (opcional, por defecto todos).}

\item{subir_a_bd}{por defecto TRUE, FALSE para test y cosas por el estilo (opcional).}

\item{fuentes}{por defecto marca todas las fuentes, pero se puede elegir una o varias de las disponibles en el README. (opcional)}
}
\value{
Un dataframe con las noticias extraidas.
}
\description{
Esta funcion permite realizar una extraccion automatizada de noticias
de BioBio y fuentes de El Mercurio.
}
\details{
Es importante mencionar que si tiene mas de una fuente seleccionada, la
cantidad maxima de resultados se aplicara para cada una de las fuentes, es
decir, si pones max_results = 10 y tienes fuentes = "emol,guioteca,bbcl"
tendras como maximo 30 resultados.
}
\examples{
\dontrun{
noticias <- extraer_noticias_max_res("inteligencia artificial",
max_results = 20, fuentes="bbcl, emol", subir_a_bd = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_fecha_ciper.R
\name{extraer_noticias_fecha_ciper}
\alias{extraer_noticias_fecha_ciper}
\title{Extrae noticias de Ciper Chile por rango de fechas}
\usage{
extraer_noticias_fecha_ciper(search_query = NULL, fecha_inicio, fecha_fin)
}
\arguments{
\item{search_query}{El termino de busqueda (opcional).}

\item{fecha_inicio}{Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}

\item{fecha_fin}{Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}
}
\value{
Un dataframe con las noticias extraidas, estandarizado al formato de datamedios.
}
\description{
Esta funcion se conecta a la API de Ciper para descargar noticias dentro de un rango de fechas.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-emol-helpers.R
\name{helper_extraer_temas}
\alias{helper_extraer_temas}
\title{Helper para extraer y procesar los temas de datos de Emol}
\usage{
helper_extraer_temas(
  source_seccion,
  source_subseccion,
  source_temas,
  fuente_actual
)
}
\arguments{
\item{source_seccion}{El contenido de la columna `_source.seccion`.}

\item{source_subseccion}{El contenido de la columna `_source.subSeccion`.}

\item{source_temas}{El contenido de la columna `_source.temas`. Puede ser
un dataframe, lista, vector, o NULL.}

\item{fuente_actual}{Un string que indica la fuente (e.g., "mediosregionales").}
}
\value{
Una lista que contiene un unico elemento: un vector de strings (character)
  con los temas procesados y unicos. Si no hay temas, devuelve una lista
  conteniendo un vector de caracteres vacio (`character(0)`).
}
\description{
Esta funcion interna procesa las columnas `_source.seccion`,
`_source.subSeccion` (si aplica), y `_source.temas` de una fila de
datos de Emol para generar una lista de temas procesados.
Los temas se convierten a minusculas y los espacios se reemplazan por guiones.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_fecha.R
\name{extraer_noticias_fecha}
\alias{extraer_noticias_fecha}
\title{Extraccion de noticias de medios chilenos por rango de fechas}
\usage{
extraer_noticias_fecha(
  search_query = NULL,
  fecha_inicio,
  fecha_fin,
  subir_a_bd = TRUE,
  fuentes = "todas"
)
}
\arguments{
\item{search_query}{Una frase de busqueda (opcional). Si la fuente es 'ciper', puede ser NULL.}

\item{fecha_inicio}{Fecha de inicio del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}

\item{fecha_fin}{Fecha de fin del rango de busqueda en formato "YYYY-MM-DD" (obligatoria).}

\item{subir_a_bd}{por defecto TRUE, FALSE para test y cosas por el estilo (opcional).}

\item{fuentes}{es un string con las fuentes a extraer. Puede ser bbcl o las de emol.}
}
\value{
Un dataframe con las noticias extraidas.
}
\description{
Esta funcion permite realizar una extraccion automatizada de noticias de
BioBio o Los medios de Emol, utilizando un rango de fechas.
}
\examples{
\dontrun{
noticias <- extraer_noticias_fecha("delincuencia", "2025-04-25",
"2025-04-28", subir_a_bd = FALSE, fuentes="bbcl")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agregar_datos_unicos.R
\name{agregar_datos_unicos}
\alias{agregar_datos_unicos}
\title{Agregar datos unicos a una tabla MySQL}
\usage{
agregar_datos_unicos(data)
}
\arguments{
\item{data}{Un data frame con los datos a insertar.}
}
\value{
No retorna ningun valor.
}
\description{
Esta funcion agrega datos a una tabla MySQL utilizando endpoints que esperan datos en formato JSON.
}
\examples{
\dontrun{
# Agregar datos unicos
noticias <- extraer_noticias_max_res("tesla", max_results=10, fuentes="bbcl", subir_a_bd = FALSE)
agregar_datos_unicos(noticias)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iteracion_emol.R
\name{iteracion_emol}
\alias{iteracion_emol}
\title{Inicializa una solicitud a emol.com y retorna maximo 10 noticias}
\usage{
iteracion_emol(search_query, page = 0, fuentes = "emol-todas")
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}

\item{page}{La pagina de busqueda para iterar, es un int}

\item{fuentes}{Es un string que deberia tener solo fuentes de emol posibles separadas por comas.}
}
\value{
Un dataframe con el caso de la busqueda, incluyendo solo columnas especificas.
}
\description{
Esta funcion auxiliar llama a emol.com utilizando una frase de busqueda.
Entrega como maximo 10 resultados. Se debe llamar desde otras funciones solo
con una fuente a la vez, es decir, sin llamar a emol-todas.
}
\examples{
\dontrun{
primer_caso <- iteracion_emol("Boric", fuentes="emol-todas")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init_req_emol.R
\name{init_req_emol}
\alias{init_req_emol}
\title{Inicializa una solicitud a emol.com y retorna el primer caso de busqueda}
\usage{
init_req_emol(search_query, fuentes = "emol-todas")
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}

\item{fuentes}{Un string donde se ponen las fuentes de emol a consultar}
}
\value{
Un dataframe con el primer caso de la busqueda.
}
\description{
Esta funcion permite realizar una consulta inicial a emol.com utilizando una frase de busqueda.
}
\examples{
\dontrun{
primer_caso <- init_req_emol("Boric", fuentes="emol")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scrape_body_ciper.R
\name{scrape_body_ciper}
\alias{scrape_body_ciper}
\title{Scrapea el cuerpo de las noticias desde una lista de URLs de Ciper}
\usage{
scrape_body_ciper(news_df, parallel = TRUE)
}
\arguments{
\item{news_df}{Un dataframe que debe contener una columna ''url''.}

\item{parallel}{Booleano que indica si se debe usar procesamiento en paralelo. Por defecto es TRUE.}
}
\value{
El dataframe de entrada con la columna ''contenido_limpio'' rellenada con el texto scrapeado.
}
\description{
Esta funcion itera sobre un dataframe de noticias, visita la URL de cada una,
y extrae el texto principal usando un selector CSS especifico para Ciper.
Admite ejecucion en paralelo para acelerar el proceso.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extraer_noticias_max_res_emol.R
\name{extraer_noticias_max_res_emol}
\alias{extraer_noticias_max_res_emol}
\title{Extraccion de noticias de Emol.com}
\usage{
extraer_noticias_max_res_emol(search_query, max_results = NULL, fuente)
}
\arguments{
\item{search_query}{Una frase de busqueda (obligatoria).}

\item{max_results}{Numero maximo de resultados a extraer (opcional, por defecto todos).}

\item{fuente}{Fuente de emol para iterar (obligatoria).}
}
\value{
Un dataframe con las noticias extraidas.
}
\description{
Esta funcion permite extraer noticias de las fuentes de Emol, tanto de las
noticias no pagas de emol, como de guioteca y los medios regionales de El Mercurio
}
\examples{
\dontrun{
noticias <- extraer_noticias_max_res_emol("inteligencia artificial", "2025-01-01",
"2025-02-24", fuente="mediosregionales")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limpieza_notas.R
\name{limpieza_notas}
\alias{limpieza_notas}
\title{Funcion para limpiar notas de contenido HTML}
\usage{
limpieza_notas(datos, sinonimos = c())
}
\arguments{
\item{datos}{Data frame donde estan almacenadas las notas}

\item{sinonimos}{Un vector de character}
}
\value{
Un dataframe con el contenido limpio en la columna contenido_limpio
}
\description{
Esta funcion permite limpiar por completo las notas eliminando codigos y secciones irrelevantes.
Verifica que el input sea un data frame con una columna llamada `contenido`.
}
\examples{

\donttest{
datos <- extraer_noticias_max_res("inteligencia artificial", max_results= 150, subir_a_bd = FALSE)
datos_proc <- limpieza_notas(datos, sinonimos = c("IA", "AI"))
}
}
